/*
 *  reaction.h
 *  hierarchical_erleap
 *
 *  Created by David  Orendorff on 9/8/08.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

/*
 * class Reaction extends the abstract class ReactionSet. Reaction represents a SINGLE reaction.
*/


#ifndef REACTION_H
#define REACTION_H

#include <list>
#include <string>


#include "reaction_set.h"
//#include "../state/state.h"

#include <boost/foreach.hpp>

using namespace std;

class ReactionMember;


namespace hierleap{
class Species;

// the represents the base-case (1 single reaction)
class Reaction : public ReactionSet {

public:
	Reaction(Reactions *parent, double rate) ;
	
	Reaction(Reaction *r) : ReactionSet(NULL, true), _changesPropensityFor(NULL) {
		_reactantList = r->_reactantList;
		_productList = r->_productList;
		_rate = r->_rate;
	}
	
	Reaction():_rate(0), ReactionSet(NULL, true) { assert(false);}
	
	~Reaction();
	
	// modifies state s.t. we want to compute upper bound for all species, and modify state to show this
	void assumeDiffusionInto(State* state, int events);
	
	double rate(){ return _rate; }
	
	// this probably shouldn't be called like this
	void _setRate(double rate){ assert(rate >=0); _rate = rate;}
	
	// Amount that this reaction will change speciesUniquId
	int delta(int speciesUniquId);
	
	double D(State *state);

	// calculates D by looking up the species type for each reactant
	double DbyType(vector<int> & state);

	// calculate D by looking up unique ID for each reactant species 
	//	IE: state contains EVERY species value in system
	virtual double DbyUniqueId(vector<int> & state);
	
	// TODO probably not the best name :)
	// Calculate D except if parent fires find Dmax for L-1 reactions
	double highDByUniqueIdNoParentRxn(vector<int> &s, int nParentRxns);

	// are enough reactants present is the state able to carry out this reaction?
	bool isViable(vector<int> & s);
	
	bool isViable(State *s);
	
	// Execute the state changes caused by apply this reaction to state
	// returns FALSE iff firing results in a negative state.
	bool fire(State *state);
	
	void unFire(State *state);
	
	bool fire(vector<int> & state);
	
	void unFire(vector<int> & state);

	// Same as fire, but does NOT affect the product of the reaction, only reactants
	void fireReactantsOnly(State *state);
	
	void addReactant(Species *s, unsigned int stoichiometry) ;
	
	void addProduct(Species *s, unsigned int stoichiometry) ;
	
	int getReactantId(int rxnIdx);
	
	int getProductId(int ridx);
	
	int getReactantType(int rxnidx);

	int getReactantStoic(int rxnidx);

	int getProductStoic(int rxnidx);

	int getProductType(int rxnidx);
	
	int numProducts(){ return _productList.size(); }

	int numReactants(){ return _reactantList.size(); }

	// the reactionset parent of the gridcell with the LHS diffusion reactant
	Reactions *inpParent();
	
	// the reactionset parent of the gridcell with the RHS diffusion reactant
	Reactions *outParent();
	
	char* myType();
	
	// Are the input and output of this reaction fully contained in root?
	bool selfFullyContainedIn(Reactions *root);
	
	// After the this reaction fires, the propensities of returned [Reaction] may change
	vector<Reaction*> * changesPropensityFor();
	
	// Do I share a common species with r?
	bool isNeighbor(Reaction* r){
		BOOST_FOREACH(ReactionMember *rm, _reactantList){
			if (r->involves(rm))
				return true;
		}
		BOOST_FOREACH(ReactionMember *rm, _productList){
			if (r->involves(rm))
				return true;
		}
		return false;
	}
	
	// Does this reaction involve r?
	bool involves(ReactionMember *r);
	
	// Maps the specie objects this reaction uses to those found s
	void updateSpeciesInfo(State *s);
	
	// A list of specie ids that change after this fires.
	vector<int> & speciesWhichChange();
	
	virtual void setupInteractingRxn(State *s);
	
	// return list of rxn with propensities which may be affected by *this* firing.
	vector<Reaction*> & maybeEffectedRxn();
protected:

	void _specialPrint(State *s, char format);
	int _uncachedDelta(int speciesId);
	
	vector<int> _speciesWhichChange;
	vector<Reaction*> _maybeEffectingRxn; // be sure to include *this*
	vector<ReactionMember*> _reactantList;
	vector<ReactionMember*> _productList;
	vector<Reaction*> * _changesPropensityFor;
	
	double _rate;
	vector<int> _deltas;

	
};

} // end namespace

#endif
