/*
 *  reaction_set.h
 *  hierarchical_erleap
 *
 *  Created by David  Orendorff on 8/15/08.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef REACTION_SET_H
#define REACTION_SET_H

#include <assert.h>
//#include <string.h>
//#include <stdio.h>
#include <queue>
//#include <math.h>

#include <boost/foreach.hpp>

using namespace std;

////#include "reactions.h"
//#include "reaction.h"
#include "../globals.h"
#include "../utils/common.h"
//#include "../state/state.h"

namespace hierleap{
	class Reaction;
}
//using namespace hierleap;

class Reactions;
class State;
template <class T> class ReactionPermutation; // this needs to be declared here...



// abstract class
// implied constraint: _dependents == null IFF type(this)==Reaction
class ReactionSet {

public:

	/////////////////////////////////// SETUP FUNCTIONS /////////////////////////////////////////
	
	ReactionSet * asReactionSet() { return this; }
	
	virtual ~ReactionSet(){
	}
	
	void _reconnectNbrs();
	
	// kinda dumb, need this to sort species.
	bool operator<(ReactionSet & rs) {
		return getUniqueId() < rs.getUniqueId();
	}
	
	void _bubbleUpBubbleId(int bId);
	
	int bubbleId; // just used to setup MCA building
	// simple idea, 'bubble up' some id num, have neighbors bubble up until their parent has that id tag
	// running time (for each node) = O(|nbrs| * log(n))
	void buildNbrsMCA();
	
	virtual void setupInteractingRxn(State *s);
	
	// automatically looks for duplicates
	void _addNbrMCA(ReactionSet *s);

    void setCoordinate(int x, int y, int z);
    
    // Get cell (x,y,z) coordinates
    // Only valid for Reaction or base cell Reactions
    void getCoordinate(int &x, int &y, int &z);
	
	void printCoordinates() {
		printf("coord: <%d, %d, %d>\n", _x, _y, _z);
	}

	void registerMe(int newId){
		if(_myUniqueId >= 0){ 
			printf("WARNING, reregistering reaction for ");
			printf("(id:%d, %d) registration.\n", _myUniqueId, newId);
		}else{
			_myUniqueId = newId;
		}
	}
    
    // name yourself and the children (not required), and register for a name
    void idSystem(char *myAddressId);

    // Returns how many reaction/s (including self) are in the tree
    int numRxnSets();

    // Returns height (including self) for the tree.
    // Assumes children are all of same height
    int height();
    
    /////////////////////////////////////////
    //////// DEBUGGING FUNCTIONS ////////////

    // format specifies which properties to print. Each node gets exactly 1 line.
    // key: e -> num events. c -> numchildren. d -> D. f -> Dfloor. h -> Dceil.
    void print(State *state, char *format);

    //////// END DEBUGGING FUNCTIONS ////////////
    /////////////////////////////////////////////


    /////////////////////////////////////////////
    ///////////////// ACCESSORS /////////////////
    
    char *myName(){
    	return _myId;
    }
    
    // Return true if *this* is a direct decendent of root
    bool grandchildOf(Reactions *root);
    
    Reactions *getParent();

    void setParent(Reactions *r);
    
    void voidParent(){
    	_parent = NULL;
    }
	
	Reactions* getRoot();

	void changeParent(Reactions *newParent);
	
	void removeDependent(ReactionSet *rxnSet);
	
    // make path from here to parent, node reached right before hitting parent is returned=
    ReactionSet *highestParentUnder(Reactions *root);

    vector<ReactionSet*> & getParentNeighbors();

    vector<ReactionSet*> & getNeighborsMCA(){
        // get most common ancestor nbrs
        return _nbrsMCA;
    }

    vector<ReactionSet*> & getNeighbors(){
        return _nbrs;
    }
    
    void clearNeighbors(){
    	_nbrs.clear();
    	BOOST_FOREACH(ReactionSet *r, getDependents()){
    		r->clearNeighbors();
    	}
		assert(_nbrs.size()==0);
    }

    void addNeighbor(ReactionSet *r){
		assert(r);
        // make sure r is not already in _nbrs
    	if (contains(_nbrs, r) || r==this) return;
        _nbrs.push_back(r);
    }

	// removes r from neighbors list. assert is thrown if not found.
	void removeNeighbor(ReactionSet *r) {
		assert(r);
		for (int i = 0; i < _nbrs.size(); i++){
			if (_nbrs[i] == r) {
				_nbrs.erase(_nbrs.begin()+i);
				return;
			}
		}
		assert(false);
	}
	
	// removes 'from' form the nieghbors list and inserts to if it does
	// not already exist.
	void changeNeighbor(ReactionSet * from, ReactionSet * to) {
		assert(from); assert(to);
		removeNeighbor(from);
		addNeighbor(to);
	}
	
	bool isReaction() { return _isReaction; }

    vector<ReactionSet*> & getDependents(){
        return _dependents;
    }

    void addDependent(ReactionSet *r);

    int getUniqueId(){ return _myUniqueId; }
    
	Reactions* asReactions();
	
	hierleap::Reaction* asReaction();

	// What index is *this* located in dependents of this->parent
	int indexInParent();
	
	// public version
	int numDependents(int rxnUniqueId){
		int deps = _numDependents(rxnUniqueId);
		if(deps < 0){
			assert(false);
		}
		return deps;
	}
	
	int numDependents() {
		return _dependents.size();
	}
	
	int smallestChildId() {
		assert(!isReaction());
		if (_smallestChildId == -1) {
			_smallestChildId = INT_MAX;
			BOOST_FOREACH(ReactionSet *r, getDependents()) {
				_smallestChildId = min(r->getUniqueId(), _smallestChildId);
			}
		} 
		return _smallestChildId;
	}

	
    /////////////// END ACCESSORS ///////////////
    /////////////////////////////////////////////


    ////////////////////////////////////////
    ////// VIRTUAL PUBLIC FUNCTIONS ////////

    virtual double D(State *state) = 0;
    
    // Reactants are accessed according to species unique ids.
    virtual double DbyUniqueId(vector<int> & s) = 0; 
    
    ///// END VIRTUAL PUBLIC FUNCTIONS //////
	/////////////////////////////////////////
	
	
	// Only call this on root!
	void makeIdsTopological(){
		assert(getParent()==NULL);
		int newId = 0;
		
		queue<ReactionSet*> regQ;
		regQ.push(this);
		
		while(!regQ.empty()){
			ReactionSet *r = regQ.front();
			regQ.pop();
			r->registerMe(newId);
			_orderedProgeny.push_back(r);
			newId++;
			
			// put children into queue
			if (!r->isReaction()){
				BOOST_FOREACH(ReactionSet *rc, r->getDependents()){
					regQ.push(rc);
				}
			}
		}
	}
			
	void assertIdsTopological() {
		assert(getParent()==NULL);
		assert(_orderedProgeny.size() >= 1);
		
		// Assert everything was put together properly. 
		for (int i = 0; i < _orderedProgeny.size(); i++) {
			assert(i == _orderedProgeny[i]->getUniqueId());
		}
	}
	
	// This may only be called on the Root ReactionSet.
	// The index of the ith ReactionSet will be the RS with i as the uniqueId.
	// Root (id: 0) is included. Asserts makeIdsTopological must be called first.
	vector<ReactionSet*> & getOrderedProgeny() {
		assert(_orderedProgeny.size() != 0);
		return _orderedProgeny;
	}
	

protected:
	
	const bool _isReaction;
    int _myUniqueId; 
	char _myId[100];
	int _idxInParent;
	int _smallestChildId;
	vector<hierleap::Reaction*> _baseNeighbors; 	// Reaction's which are contained in parent, not contained by self and are nbrs to
						// some reaction in self

	////////////////////////////////////////////
	////// INIT and SETUP vars		////////////
	int _x, _y, _z; // coordinates
	
	vector<ReactionSet*> _nbrsMCA; // MCA - most common ancestor neighbors. ie: first possible (grand)parent of *this* is parent of nMCA[i]
	
	////// END INIT and SETUP vars	////////////
	////////////////////////////////////////////
	
	vector<ReactionSet*> _orderedProgeny; 

	vector<ReactionSet*> _nbrs; // which reactionSets on the *same level* are connected by some reaction?
	
	// dependents, ie children below this reactionSet node
	vector<ReactionSet*> _dependents;
	
	ReactionSet(Reactions* parent, bool isReaction);
	
	void _buildDRL(vector<hierleap::Reaction*> & listSoFar);
	
	// for reaction id somewhere under here, return how many dependents it has
	int _numDependents(int rxnUniqueId){
		if (rxnUniqueId == getUniqueId()){
			return _dependents.size();
		}
		if (isReaction()) return -1;

//		BOOST_FOREACH(ReactionSet *r, _dependents){
//			int deps = r->_numDependents(rxnUniqueId);
//			if (deps>=0) return deps;
//		}
		for (int i=0; i < _dependents.size(); i++){
			int deps = _dependents[i]->_numDependents(rxnUniqueId);
			if (deps>=0) return deps;
		}
		return -1;
	}

	virtual void _specialPrint(State *s, char format) = 0;
   
private:
	Reactions *_parent;
};


#endif
