/*
 *  reaction_map.h
 *  gpu_hierleap
 *
 *  Created by David Orendorff on 11/28/11.
 *  Copyright 2011 UC Irvine. All rights reserved.
 *
 */

#ifndef REACTION_MAP
#define REACTION_MAP

#include <vector>

#include "reaction_set.h"

using namespace std;

class ReactionSet;

// A specialized map from Reaction* to Value
template <class Value>
class ReactionSetMap {
private:
	vector<Value> _mapping;
	vector<char> _count;
	
	// makes it so idx may be referenced into mapping and count safely
	void _enlarge(int newSize) {
		assert(newSize > _mapping.size());
		_mapping.resize(newSize, NULL);
		_count.resize(_mapping.size(), NULL);
	} 
	
	void _set(ReactionSet *r, Value v);
	
public:
	ReactionSetMap() {}
	ReactionSetMap(int capacity) { _enlarge(capacity); }
	
	void rehash(int doNothing) {}
	
	Value & operator[] (ReactionSet* r);
	
	inline Value get(ReactionSet *r) {
		assert(r->getUniqueId() >= 0 && r->getUniqueId() < _mapping.size());
		return _mapping[r->getUniqueId()]; 
	}
	
	void set(ReactionSet *r, Value v){
		int idx = r->getUniqueId();
		if (idx >= size()) {
			_enlarge(max((idx + 1) * 2, 10));
		}
		_count[idx] = 1;
		_mapping[idx] = v;
	}
	
	int count(ReactionSet *r) {
		int idx = r->getUniqueId();
		if (idx >= size())
			return 1;
		return _count[idx];
	}
	
	// This is more like capacity.
	int size(){
		return _mapping.size();
	}
};

#endif