/*
 *  reactions.h
 *  hierarchical_erleap
 *
 *  Created by David  Orendorff on 9/8/08.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef REACTIONS_H
#define REACTIONS_H


class Reactions;
//class ReactionSet;

namespace hierleap{
class Reaction;
class Species;
}
using namespace hierleap;

template <class Rxn>
class ReactionPermutation;

#include "reaction_set.h"

// this represents a 'group' of reactions
class Reactions : public ReactionSet {

public:

	Reactions(Reactions *parent) : ReactionSet(parent, false), _rpCache(NULL) {
	}

	// Things that will eventually need to be calculated and cached are cached now.
	void preCache();
	
	
	// This is basically a glorified DFS. An edge is between two cells if they have 
	// species in common AND they have have reaction events.
	// Also called "connected components" in hierleap paper.
	void connectedCellsWithEvents(vector<Reactions*> & explored, State *s);
	
	// Return a vector of the species represented in this cell from state
	// 		note: an assertion is thrown if the children of *this* are not of type Reaction*
	vector<int> & speciesXByType(State *s);
	
	vector<int> & propensityEffectedByTheseSpecies();
	
	vector<int> & speciesSandbox(int nSpecies) {
		if (_speciesSandbox.size() == 0) {
			_speciesSandbox = vector<int>(nSpecies, NULL);
		}
		assert(nSpecies == _speciesSandbox.size());
		return _speciesSandbox;
	}
	
	ReactionPermutation<Reaction*> * reactionPermutationSandbox();

	// Returns a vector of type Reaction* which have self as a (grand)parent
	vector<Reaction*> & descendantReactionList();
	
	// Returns all bottom-level cells under this->parent but not 
	//		under *this* which *this* events may diffuse into
	// Build on demand.
	vector<Reactions*> & baseNeighbors();
	
	void addDependent(ReactionSet *r);
	
	double D(State *state);
	
	double DbyUniqueId(vector<int> & s);
	
	// What is maximum a species involving Reactions may change 
	int maxStoicDelta(int speciesUniqueId);

	// pessimistically modifies state under the assumption that 'events' diffusion events 
	// happens to *this* cell. 
	// Should only be called on a base level cell
	void assumeDiffusionInto(State *state, int events);
	
	void addContainedSpecies(Species *s);
	
	void clearContainedSpecies(){
		_childSpeciesUniqueIds.clear();
		BOOST_FOREACH(ReactionSet *r, getDependents()){
			if (!r->isReaction())
				r->asReactions()->clearContainedSpecies();
		}
	}
	
	// if all dependents are of type Reaction*
	bool onlyContainReaction(){
		assert(!isReaction());
		BOOST_FOREACH(ReactionSet *r, getDependents()){
			if (!r->isReaction()) return false;
		}
		return true;
	}
	
	// look at all base-cells, see who is neighbors with the most other base cells (function of dimmension)
	int maxBaseCellNumNeighbors(){
		if (onlyContainReaction()){
			return getNeighbors().size();
		}
		int maxNbr=0;
		BOOST_FOREACH(ReactionSet *r, getDependents()){
			maxNbr = max(maxNbr, r->asReactions()->maxBaseCellNumNeighbors());
		}
		return maxNbr;
	}
	
	int _whatIndexChild(ReactionSet *r){
		vector<ReactionSet*> deps = getDependents();
		for (int i=0; i < deps.size(); i++){
			if (deps[i] == r) return i;
		}
		assert(false); return -1;
	}
	
	// Does this reactionSet have any species in common with r?
	bool isNeighbor(Reactions *r);

protected:

	// assumes setEvents has already been called on this node
	//void _rxnSample(ReactionPermutation *rp);

	// if the format is undefined for this level, see if the specific nodes can handle it
	void _specialPrint(State *s, char format){
		switch (format){
			case 'c' : printf(" children: %d", (int)_dependents.size()); break;
		}
	}

	vector<Reactions*> _baseNeighbors;
	vector<int> _pebts; // species that the propensity for this Reactions depends upon.
	
	vector<int> _speciesSandbox;
	vector<int> _childSpeciesUniqueIds;
	vector<Reaction*> _drl;
	ReactionPermutation<Reaction*> * _rpCache;
};

#endif
