/*
 * build_reaction_topology.h
 *
 *  Created on: Sep 11, 2009
 *      Author: dweo
 */

#ifndef BUILDREACTIONTOPOLOGY_H_
#define BUILDREACTIONTOPOLOGY_H_

#include <queue>

#include "../reaction.h"
#include "../../state/species.h"

using namespace hierleap;

using namespace std;

// Create a bunch of reactions and complementary species
//	The 'template' from *oneCell* will be duplicated in a XxYxZ grid
//	with diffusion reactions having rate *diffusionConst*
class BuildReactionTopology {
public:

	// Primary function of this class. "Return values" are the pass by reference
	//	parameters at the end of function signature.
	void build(vector<Reaction*> & oneCell, vector<Species*> & species,
						int levels, int x, int y, int z, bool diffusion, double diffusionRate, int cellsPerMetaCell,
								State * &stateReturnPtr, Reactions * &rootReturnPtr);
	
	// Similar to "build" but root ends up with |Reaction| children.
	void buildFlat(vector<Reaction*> & oneCell, vector<Species*> & cellSpecies, int levels,
													int x, int y, int z, bool diffusion, double diffusionRate,
													State * &stateReturnPtr, Reactions * &rootReturnPtr);
	
	// Does some inefficient O(n^2) checks to make sure everything looks reasonably consistent.
	void assertConsistency(Reactions* root, State *state);
	
private:

	// Connect neighbors, one to another
	void _connectRxnNeighbors(vector<Reaction*> & rs);
	void _connectRxnsNeighbors(vector<ReactionSet*> & rs);
	// Create new vector with everything a Reactions*. Throws assert if there is a Reaction* involved.
	vector<Reactions*> asReactionsVector(vector<ReactionSet*> v);
	
	void _myReactionsCreator(vector<Reaction*> & cellReactions, vector<Species*> & cellSpecies,
												int x, int y, int z, bool diffusion,
												float diffusionRate, vector<ReactionSet*> & rxnSets, vector<Species*> & species);

	Reactions* _hierchyCreator(vector<ReactionSet*> & rxns, int aveChildRxns);
	
	// Creates new cells that contain |cellsMergedPerGroup| of the origional cells.
	vector<ReactionSet*> _mergeSomeCells(vector<ReactionSet*> & cells, 
										 vector<Species*> & species, int cellsMergedPerGroup);
	
	// Used in buildFlat. Fixes problems with relationships.
	void _reassignSpeciesReactionsRelationships(vector<Species*> species, Reactions *root);
	
	// Root is one level, r is one level. No return value because r is added to root as (sub) dependent.
	void _connectFlatRxnToRoot(Reactions *root, ReactionSet *r, int levelToAdd);
};

#endif /* BUILDREACTIONTOPOLOGY_H_ */
