/*
 *  cell.cpp
 *  hierarchical_erleap
 *
 *  Created by David  Orendorff on 1/26/09.
 *  Copyright 2009 __MyCompanyName__. All rights reserved.
 *
 */

#include "cell.h"

#include "../../utils/common.h"

using namespace hierleap;

Cell::Cell(vector<Reaction*> & cellReactions, vector<Species*> & cellSpecies, int x, int y, int z){
	_parent = new Reactions(NULL);
	_parent->setCoordinate(x,y,z);
	
	// now add reactions/species to this cell
	for (vector<Species*>::iterator s = cellSpecies.begin(); s != cellSpecies.end(); s++){
		Species *newSpecies = new Species(*s);
		newSpecies->setTypeId((*s)->getTypeId());
		newSpecies->setParent(_parent);
		newSpecies->registerMe();
		_parent->addContainedSpecies(newSpecies);
		_s.push_back(newSpecies);
	}
	
	for (vector<Reaction*>::iterator r = cellReactions.begin(); r != cellReactions.end(); r++){
		
		Reaction *newR = new Reaction(_parent, (*r)->rate());
		newR->setCoordinate(x,y,z);
	
		// make input/output species and coefficients
		
		// add reactants/products to new reaction
		// reactants
		for (int i=0; i < (*r)->numReactants(); i++){
			int type = (*r)->getReactantType(i);
			int stoic = (*r)->getReactantStoic(i);
			newR->addReactant(getSpeciesByType(type), stoic);
		}
		// products
		for (int i=0; i < (*r)->numProducts(); i++){
			int type = (*r)->getProductType(i);
			int stoic = (*r)->getProductStoic(i);
			newR->addProduct(getSpeciesByType(type), stoic);
		}
		
		_parent->addDependent(newR);
	}

	// need to add the GridcellSpecies and GridcellReaction for this cell
	
}

// diffuse species from this cell to neighbor
void Cell::addDiffusionReactions(Cell *nbrCell, float diffusionRate){
	for (vector<Species*>::iterator s = _s.begin(); s != _s.end(); s++){
		Reaction *r = new Reaction(_parent, diffusionRate);
		_parent->addDependent(r);

		// add reactant/product
		r->addReactant(*s, 1);
		r->addProduct(nbrCell->getSpeciesByType((*s)->getTypeId()), 1);

	}
}

// add c as a nieghbor
void Cell::addNeighbor(Cell *c){
	vector<ReactionSet*> & nbrs = _parent->getNeighbors();

	// meaning c is not already a neighbor
	//XXX; /// todo: determine what we mean by neighbor!
	if (!contains(nbrs,  c->parentRxnset()->asReactionSet()))  //find(nbrs.begin(), nbrs.end(), c->parentRxnset()) == nbrs.end())
		_parent->addNeighbor(c->parentRxnset());
}

