/*
 *  cell.h
 *  hierarchical_erleap
 *
 *  Created by David  Orendorff on 1/26/09.
 *  Copyright 2009 __MyCompanyName__. All rights reserved.
 *
 */

/*
 * Cell is a helping class used when the initial system is being created on a grid of cells. It is not used during simulation.
 */
#ifndef CELL_H
#define CELL_H

#include <vector>
#include <algorithm>

#include "../reaction_set.h"
#include "../reactions.h"
#include "../reaction.h"
#include "../../state/species.h"

using namespace std;
using namespace hierleap;

// used to help organize the creation of the system
class Cell{
public:
	
	Cell(){
		// should never use a class if this has been called
		_parent = NULL; // can't use the class if we can't get the parent :)
	}
	
	// copy reactions and species into this cell
	Cell(vector<Reaction*> & cellReactions, vector<Species*> & cellSpecies, int x, int y, int z);	
	
	Reactions *parentRxnset(){ return _parent;}

	Species* getSpeciesByType(int typeId){
		assert(typeId < (int)_s.size());
		return _s[typeId];
	}

	// diffuse species from this cell to neighbor
	void addDiffusionReactions(Cell *nbrCell, float diffusionRate);
	
	// add c as a nieghbor
	void addNeighbor(Cell *c);
	
	vector<Species*> & getSpecies(){ return _s;}

protected:
	vector<Species*> _s;
	Reactions *_parent; // parent to all reactionsets here
};

#endif
