# There exist some parameters for this model which result in a failure for CaliBayes.
# Find and analyse these cases.

from glob import glob
from math import exp, sqrt
import os
import sys
from numpy import *

xmlTemplate="""<?xml version="1.0" encoding="UTF-8"?>
<sbml xmlns="http://www.sbml.org/sbml/level2" level="2" version="1">
  <model id="ImmigrationDeath01" name="Immigration-Death (002), variant 01">
    <listOfUnitDefinitions>
      <unitDefinition id="substance">
        <listOfUnits>
          <unit kind="item" multiplier="1" offset="0"/>
        </listOfUnits>
      </unitDefinition>
    </listOfUnitDefinitions>
    <listOfCompartments>
      <compartment id="Cell"/>
    </listOfCompartments>
    <listOfSpecies>
      <species id="Q" compartment="Cell" initialAmount="%(initial)d" hasOnlySubstanceUnits="true"/>
    </listOfSpecies>
    <listOfParameters>
      <parameter id="Alpha" value="%(alpha)f"/>
      <parameter id="Mu" value="%(deathRate)f"/>
    </listOfParameters>
    <listOfReactions>
      <reaction id="Immigration" reversible="false">
        <listOfProducts>
          <speciesReference species="Q"/>
        </listOfProducts>
        <kineticLaw>
          <math xmlns="http://www.w3.org/1998/Math/MathML">
            <ci> Alpha </ci>
          </math>
        </kineticLaw>
      </reaction>
      <reaction id="Death" reversible="false">
        <listOfReactants>
          <speciesReference species="Q"/>
        </listOfReactants>
        <kineticLaw>
          <math xmlns="http://www.w3.org/1998/Math/MathML">
            <apply>
              <times/>
              <ci> Mu </ci>
              <ci> Q </ci>
            </apply>
          </math>
        </kineticLaw>
      </reaction>
    </listOfReactions>
  </model>
</sbml>"""

# for each t in times, generate the expeced value given these parameters.
def genExpectedValues(times, alpha, deathRate, initial):
    ks = alpha
    kd = deathRate
    def f(t):
        return exp(-kd*t) * (kd*initial +(-1+exp(kd*t))*ks) / kd
    return map(f, times)

def genStandardDeviation(mean, times, alpha, deathRate, initial):
    ks = alpha
    kd = deathRate
    m = initial
    def firstMoment(t):
        return exp(-kd*t) * (kd*initial +(-1+exp(kd*t))*ks) / kd
    def secondMoment(t):
        return exp(-2*kd*t) * (kd**2 * (-m+1)*m + 2*kd*(-1+exp(kd*t))*m*ks + (-1+exp(kd*t))**2*ks**2) / kd**2
    def st(idx):
        t = times[idx]
        vr = exp(-2*kd*t)*(-1+exp(kd*t))*(kd*m+exp(kd*t)*ks)/kd
        return sqrt(vr)
        return sqrt(secondMoment(times[idx]) + mean[idx] - mean[idx]**2)
    assert(times[0]==0)
    stds = [1] + map(st, range(1, len(times)))
    return stds

runCmd='../build/Release/gpu_hierleap -f %(fileName)s -r 20 -t 50 -p 1 -o out -l 4 -m 20 -c None -a HIER -x 100 --seed 22'

# runs hierleap on file specified by filename. 
# (times, Returns list of trajectories)
def runSim(fn):
    ret = os.system(runCmd % {'fileName':fn})
    if ret != 0:
        print 'FAILED'
        print runCmd % {'fileName':fn}
        assert(False)
    x=[]
    for outFile in glob('out/Q*.txt'):
        f = file(outFile)
        lns = f.readlines()
        f.close()
        
        def getTrajectory(ln):
            return map(float, ln.split())
        x.extend(map(getTrajectory, lns))
        os.remove(outFile)
        
    times = range(0, 51)
    return (times, x)



def xNormed(x, params, times):
    trueMean = genExpectedValues(times=times, **params)
    trueStd = genStandardDeviation(mean=trueMean, times=times, **params)
    mu = mean(array(x), 0)
    n = len(times)
    assert(len(mu)==n)
    return map(lambda (i,v): sqrt(len(x))*(mu[i] - trueMean[i]) / trueStd[i], enumerate(mu))
    
#results = {(0.13, 3): [3.143227687279921], (1, 3): [3.5373247895361097], (0.1, 7): [], (0.17, 10): [3.0869745325651583, 3.0869745325651579, inf, inf, inf, inf, inf, inf, inf, -inf, inf, -inf, inf, inf, inf], (5, 0.17): [], (10, 0.13): [], (0.01, 3): [4.4032478620684925, 8.5784725255316676, 8.2011262189164533, 5.1616191287613056, 4.3892724724614132, 4.3893757161035403, 4.3893808563735242, 3.6147844441384147, 3.614784456460256, 3.6147844564602569, 4.3893811257017381, 3.6147844564602578, 4.3893811257017381, 4.3893811257017381, 5.1639777949432224, 6.7131711334261883, 5.1639777949432224], (0.17, 3): [], (0.1, 10): [4.9923231932575307, 4.4721359549995805, 3.5777087639996634, -inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf], (0.03, 7): [3.8415629044878958, 3.1334179832807307, 3.9036002130736005, 3.9036002917940675, 3.2204702407301595, 3.22047024073016, 3.2204702407301595, 3.2204702407301595], (10, 5): [], (2, 1): [], (100, 0.17): [], (0.1, 3): [], (0.03, 3): [7.07962217051743, 3.130495168499706], (5, 1): [], (2, 5): [], (10, 3): [], (100, 0.1): [], (100, 10): [inf, inf, -inf, -inf, inf, -inf, inf, inf, -inf, inf, inf, inf, -inf], (100, 7): [], (100, 5): [], (5, 5): [], (5, 0.1): [], (100, 1): [], (1, 0.13): [3.3616041321868226, 3.4999091896340579, 3.3539567651477578], (100, 0.13): [], (1, 0.17): [], (1, 5): [3.0129725789954196], (10, 0.1): [], (1, 0.1): [], (0.01, 5): [11.314722872444053, 14.856229932518874, 10.999227137276684, 3.0000000000000013, 4.9999999999999982, 5.0000000000000009, 4.0000000000000009, 3.0000000000000004], (5, 7): [], (1, 10): [inf, inf, inf, inf, inf, -inf, inf, -inf, inf, -inf, -inf, -inf], (2, 0.1): [], (0.13, 5): [], (0.1, 0.17): [], (0.1, 5): [], (1, 1): [], (0.17, 0.1): [], (0.01, 1): [3.956176723484425, 3.1897615872361751, 3.4379998528979216, 4.0176303749117226, 3.1279965634249627, 4.4721359549995414, 3.1304951684996936, 3.5777087639996634], (0.17, 0.17): [], (0.17, 5): [], (0.03, 5): [7.60599236968888, 5.7412265058171732, 4.0414518842697467, 3.4641016151377548, 3.4641016151377548, 4.0414518843273806], (0.1, 0.1): [], (0.13, 1): [], (10, 0.17): [], (0.1, 1): [], (0.13, 10): [inf, inf, inf, inf, inf, inf, inf, inf, inf, -inf, inf, -inf, inf], (0.03, 0.1): [], (0.17, 1): [], (0.03, 1): [4.0920839953085775, 3.4043059126517363], (0.01, 10): [13.495252510303882, 18.382590472033648, 4.242640654035422, 4.2426406871192857, 4.2426406871192857, 5.6568542494923797, inf, inf, inf, -inf, inf, inf, inf, inf, inf, inf, inf, inf], (2, 3): [], (10, 1): [], (2, 10): [-inf, inf, -inf, inf, inf, inf, inf, -inf, inf, inf, inf, inf, inf], (5, 0.13): [], (0.1, 0.13): [], (2, 7): [3.3705446783229926], (2, 0.17): [3.2680221158987939], (5, 3): [], (0.03, 10): [6.9324870490465615, 3.265705856206079, 3.2659863109771741, 3.2659863237109041, 3.2659863237109041, 3.2659863237109041, 5.715476066494082, 3.2659863237109041, 4.0824829046386313, 4.0824829046386304, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf], (5, 10): [-inf, -inf, inf, -inf, inf, inf, inf, -inf, inf, -inf, inf, -inf], (0.03, 0.17): [], (10, 10): [inf, -inf, inf, -inf, inf, inf, -inf, -inf, -inf, inf, -inf, -inf, -inf], (2, 0.13): [], (0.17, 0.13): [], (0.01, 0.13): [], (0.03, 0.13): [], (10, 7): [-3.1269565303776252], (0.13, 0.13): [], (100, 3): [3.100358748116451], (0.01, 0.17): [], (0.13, 7): [3.4222927858510173, 3.0941277241940703], (1, 7): [], (0.13, 0.1): [], (0.13, 0.17): [], (0.01, 7): [3.5849471950656375, 15.514413195466426, 10.141492202704228, 5.4089870175536099, 3.0425553168809163, 3.0425553170225306, 3.0425553170226602, 3.0425553170226602, 3.0425553170226602, 3.0425553170226598, 4.2257712736425832, 4.2257712736425841, 4.2257712736425841, 3.0425553170226602, 3.0425553170226594, 3.0425553170226602, 3.0425553170226602, 4.2257712736425841, 3.0425553170226602, 3.0425553170226602], (0.17, 7): [3.3616722529322076, 4.2225883177075563], (0.01, 0.1): []}
results={(0.13, 3): [3.143227687279921], (5, 0.17): [], (100, 0.13): [], (2, 5): [], (100, 1): [], (300, 1): [], (5, 5): [], (10, 7): [-3.1269565303776252], (0.01, 5): [11.314722872444053, 14.856229932518874, 10.999227137276684, 3.0000000000000013, 4.9999999999999982, 5.0000000000000009, 4.0000000000000009, 3.0000000000000004], (1, 1): [], (50, 1): [], (0.03, 5): [7.60599236968888, 5.7412265058171732, 4.0414518842697467, 3.4641016151377548, 3.4641016151377548, 4.0414518843273806], (0.1, 1): [], (0.13, 10): [inf, inf, inf, inf, inf, inf, inf, inf, inf, -inf, inf, -inf, inf], (1, 0.13): [3.3616041321868226, 3.4999091896340579, 3.3539567651477578], (50, 0.1): [], (50, 0.13): [], (0.1, 0.13): [], (0.01, 0.13): [], (0.13, 7): [3.4222927858510173, 3.0941277241940703], (0.13, 0.17): [], (5, 0.13): [], (0.01, 3): [4.4032478620684925, 8.5784725255316676, 8.2011262189164533, 5.1616191287613056, 4.3892724724614132, 4.3893757161035403, 4.3893808563735242, 3.6147844441384147, 3.614784456460256, 3.6147844564602569, 4.3893811257017381, 3.6147844564602578, 4.3893811257017381, 4.3893811257017381, 5.1639777949432224, 6.7131711334261883, 5.1639777949432224], (0.17, 3): [], (2, 1): [], (50, 7): [], (0.03, 3): [7.07962217051743, 3.130495168499706], (5, 1): [], (10, 3): [], (100, 0.1): [], (1, 5): [3.0129725789954196], (1, 10): [inf, inf, inf, inf, inf, -inf, inf, -inf, inf, -inf, -inf, -inf], (0.1, 5): [], (0.17, 0.17): [], (0.13, 1): [], (10, 0.17): [], (0.01, 10): [13.495252510303882, 18.382590472033648, 4.242640654035422, 4.2426406871192857, 4.2426406871192857, 5.6568542494923797, inf, inf, inf, -inf, inf, inf, inf, inf, inf, inf, inf, inf], (2, 7): [3.3705446783229926], (100, 7): [], (0.03, 10): [6.9324870490465615, 3.265705856206079, 3.2659863109771741, 3.2659863237109041, 3.2659863237109041, 3.2659863237109041, 5.715476066494082, 3.2659863237109041, 4.0824829046386313, 4.0824829046386304, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf], (5, 10): [-inf, -inf, inf, -inf, inf, inf, inf, -inf, inf, -inf, inf, -inf], (0.03, 0.17): [], (10, 10): [inf, -inf, inf, -inf, inf, inf, -inf, -inf, -inf, inf, -inf, -inf, -inf], (300, 7): [3.6606011736299728], (5, 7): [], (0.13, 0.1): [], (0.01, 7): [3.5849471950656375, 15.514413195466426, 10.141492202704228, 5.4089870175536099, 3.0425553168809163, 3.0425553170225306, 3.0425553170226602, 3.0425553170226602, 3.0425553170226602, 3.0425553170226598, 4.2257712736425832, 4.2257712736425841, 4.2257712736425841, 3.0425553170226602, 3.0425553170226594, 3.0425553170226602, 3.0425553170226602, 4.2257712736425841, 3.0425553170226602, 3.0425553170226602], (0.17, 7): [3.3616722529322076, 4.2225883177075563], (300, 0.13): [], (1, 3): [3.5373247895361097], (50, 3): [], (0.17, 10): [3.0869745325651583, 3.0869745325651579, inf, inf, inf, inf, inf, inf, inf, -inf, inf, -inf, inf, inf, inf], (0.03, 7): [3.8415629044878958, 3.1334179832807307, 3.9036002130736005, 3.9036002917940675, 3.2204702407301595, 3.22047024073016, 3.2204702407301595, 3.2204702407301595], (100, 0.17): [], (0.1, 3): [], (5, 0.1): [], (50, 0.17): [], (0.13, 5): [], (0.1, 0.17): [], (0.17, 0.1): [], (0.1, 0.1): [], (1, 0.17): [], (0.03, 0.1): [], (0.17, 1): [], (2, 3): [], (100, 3): [3.100358748116451], (300, 3): [], (5, 3): [], (10, 5): [], (2, 0.1): [], (2, 0.13): [], (0.17, 0.13): [], (300, 0.1): [], (0.13, 0.13): [], (1, 7): [], (50, 10): [inf, inf, -inf, inf, -inf, inf, -inf, inf, inf, inf, -inf, -inf, -inf], (0.1, 7): [], (0.1, 10): [4.9923231932575307, 4.4721359549995805, 3.5777087639996634, -inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf], (100, 10): [inf, inf, -inf, -inf, inf, -inf, inf, inf, -inf, inf, inf, inf, -inf], (100, 5): [], (300, 5): [], (1, 0.1): [], (0.03, 0.13): [], (10, 0.13): [], (0.01, 1): [3.956176723484425, 3.1897615872361751, 3.4379998528979216, 4.0176303749117226, 3.1279965634249627, 4.4721359549995414, 3.1304951684996936, 3.5777087639996634], (0.17, 5): [], (50, 5): [], (2, 10): [-inf, inf, -inf, inf, inf, inf, inf, -inf, inf, inf, inf, inf, inf], (0.03, 1): [4.0920839953085775, 3.4043059126517363], (10, 1): [], (300, 0.17): [], (300, 10): [inf, inf, inf, inf, -inf, -inf, inf, inf, -inf, -inf, -inf, inf, inf], (2, 0.17): [3.2680221158987939], (10, 0.1): [], (0.01, 0.17): [], (0.01, 0.1): []}


import numpy as np
import matplotlib.pyplot as plt
def plotResults(results):
    f = plt.figure()
    a = f.add_subplot(111)
    maxOff = max(map(len, results.values()))
    for k in results.keys():
        lbl = 'o'
        clr = '.75'
        if len(results[k]) > 3:
            clr = str((10+len(results[k])) / (maxOff+10))
            lbl = 'd'
        a.loglog(k[0], k[1], lbl, color=clr)
        #a.plot(k[0], k[1],lbl, color=clr)
        a.hold(True)
    a.set_xlabel('Immigration Rate')
    a.set_ylabel('Death Rate')
    a.set_xlim(.005, 110)
    a.set_ylim(.05, 11)
    # plt.show()
    plt.savefig('fig.png')

if __name__=='__main__':
    if len(sys.argv) > 1:
        print 'plotting results'
        plotResults(results)
        exit()

    xmlFile = 'temp.xml'
    imRate = [.01, .03, .1, .13, .17, 1, 2, 5, 10, 50, 100, 300]
    #deathRate = [.01, .1, 1, 10, 100]
    results = {}
    #imRate = [1, 10]
    deathRate = [.1, .13, .17, 1, 3, 5, 7, 10]
    params = {'initial':100}

    for ir in imRate:
        for dr in deathRate:
            k = (ir, dr)
            if k in results: 
                continue
            params['alpha']=ir
            params['deathRate']=dr
            f = file(xmlFile, 'w')
            f.write(xmlTemplate % params)
            f.close()
            
            times,x = runSim(xmlFile)
            zScore = xNormed(x, params, times)
            print zScore
            zHigh = filter(lambda n: abs(n) > 3, zScore)
            results[k] = zHigh
            if zHigh:
                '%d Z-scores were too large...' % len(zHigh)
                print zHigh
            print(str(results))
                
    print results
    os.system('echo "%s" > Zs.txt' % str(results))


