#!/bin/python

import sys
import os
import os.path
from math import log
from math import ceil
from math import sqrt
from numpy import mean, std

#import numpy as np
#import matplotlib.pyplot as plt

def plotData():    
    #from data import *
    x = map(lambda s: float(s.split()[0]), speedupVersion.split('\n'))
    y = ['safeSpeedupVersion', 'sepAcceptAllProbsInvolved', 'sepAcceptFineCoarseInvolved']
    labels= ['k', 'r', 'b', 'g']
    
    f = plt.figure()
    a = f.add_subplot(111)
    ymin = 99999
    ymax = 0
    for i, yName in enumerate(y):
        d = map(lambda s: float(s.split()[1]), (globals()[yName]).split('\n'))
        #a.plot(x, d, labels[i%(len(labels))])
        a.loglog(x, d, labels[i%(len(labels))])
        a.hold(True)
        ymin = min(ymin, min(d))
        ymax = max(ymax, max(d))
        
    a.set_xlim(min(x), max(x))
    a.set_ylim(ymin=ymin, ymax=ymax)
    a.set_xlabel('Reaction Channel Duplicates')
    a.set_ylabel('L')
    h = a.legend(y, loc='upper left') # loc=4
    # plt.set(h,'Location','Northwest')
    a.set_title('Highest L to achieve 50% acceptance')
    plt.savefig('fig.png')
    plt.show()
    
def plotCellsScaleData():
    import numpy as np
    import matplotlib.pyplot as plt
    #import data

    
#CMD = '../build/Release/gpu_hierleap -f ../tests/cases/dsmts-001-01.xml  -r 5 -t 50 -p 1 -o test_results -l 4 -c None -a HIER -m %(m)d -x %(x)d --seed 229 -F 0'
CMD = '../build/Release/gpu_hierleap -f ../tests/cases/dsmts-001-01.xml -r 20 -t 50 -p 1 -o test_results -l 12 -c None -a HIER -m %(m)d -x %(x)d --seed 29 -F 0'
# note: for pigtail used -t .2
WR_CMD = './gpu_hierleap -f pigtail.xml -r 10 -t .000001 -p .000001 -o test_results -l %(l)d --diffusion_rate 0.1 -c None -a HIER -m %(m)d -x %(x)d -y %(y)d --seed 29 -F 0'
runtimeCache = {}
leapCache = {}

# returns standard out of running command
def getOutput(cmd):
    print 'running %s' % cmd
    if os.path.exists('t'):
        os.remove('t')
    ret = os.system(cmd + ' > t')
    f = file('t','r')
    lns = f.readlines()
    f.close()
    return lns

def getAvgLeap(cmd):
    if cmd in leapCache.keys():
        return leapCache[cmd]

    lns = filter(lambda s: 'Avg_L_of_marginal_accept' in s, getOutput(cmd))
    if (len(lns) != 1):
        print 'filtered lines not of length 1...'
        print lns
    leapCache[cmd] = float(lns[0].split()[-1])
    return leapCache[cmd]

def avg(x):
    return sum(x) / float(len(x))

def getAvgRuntime(cmd):
    if cmd in runtimeCache.keys():
        return runtimeCache[cmd]

    lns = filter(lambda s: 'of duration' in s, getOutput(cmd))
    assert(lns)
    def lnToFloat(ln):
        return float(ln.split()[-2])
    runtimeMean = avg(map(lnToFloat, lns))
    runtimeCache[cmd] = runtimeMean
    return runtimeCache[cmd]

def getMinRuntime(cmd):
    lns = filter(lambda s: 'of duration' in s, getOutput(cmd))
    assert(lns)
    def lnToFloat(ln):
        return float(ln.split()[-2])
    return min(map(lnToFloat, lns))

def getRuntimeStats(cmd):
    lns = filter(lambda s: 'of duration' in s, getOutput(cmd))
    assert(lns)
    def lnToFloat(ln):
        return float(ln.split()[-2])
    times = map(lnToFloat, lns)
    assert(times)
    return (mean(times), std(times))
    

def uniqueBlockNumAndRxnsPerBlock(nRxns):
    'returns a list of tuples of distinct (blocks, nRxnsPerBlock)'
    pc = 1
    pm = nRxns
    while pc <= pm:
        while nRxns / (pc+1) == pm:
            pc+=1
            print 'pc(%d)'%pc
        while (pm-1) / nRxns == pc:
            pm-=1
            print 'pm(%d)'%pm
        yield (pc, pm)
        pc+=1
            
#def removeDuplicatesByComparer(lst, cmp=(lambda a,b:a==b)):
#    'returns a list such that for all i,j: cmp(lst[i], lst[j]) is false'
#    return [ lst[i] for i,x in enumerate(mylist) if !any(map(lambda c: cmp(c,x)), lst)]

# For each list in arg, returns
# [(a1[0], an[1][0]...an[n][0]), (a1[0], an[1][0]...an[n][1]), ...(a1[-1], a[1][-1]...
def listCombinations(a1, *an):
    if not an:
        for v in a1:
            yield (v,)
    else:
        for t in listCombinations(*an):
            for v in a1:
                yield (v,) + t

# for each val in xDim, returns a list with highestVal achievable 
#    in adjacent list over possible values of M
def optimalOverMParam(cmd, xDim, maximizationFunction=getAvgLeap, optimumFunc=max):
    params = {}
    bestVals = []
    for x in xDim:
        params['x'] = x
        n = int(sqrt(x)) # we'd like the optimal value to never be greater than n.
        n *= int(sqrt(n)) * 3
        n = min(x, n)
        m = max(1, int(sqrt(n)) / 2)
        Ms = range(m, n+1, int((n//10)+1)) # todo, this could be a bug.

        print "MS"
        print Ms
        #bestIdx = len(Ms) // 2
        bestIdx = len(Ms) - 2
        def v(idx):
            if idx < 0 or idx >= len(Ms): return -1
            d= {'x':x, 'm':Ms[idx]} 
            print d
            return maximizationFunction( cmd % d )
        # finding maximum for value at bestIdx
        vLs = map(v, range(len(Ms)))
        bestVal = optimumFunc(vLs)
        print zip(Ms, vLs)
        print 'Best cells/metacell config for x=%d results in Val=%f.' % (x, bestVal)
        bestVals.append(bestVal)
#        while not( v(bestIdx) >= v(bestIdx - 1) and v(bestIdx) >= v(bestIdx + 1) ):
#            if v(bestIdx) < v(bestIdx + 1):
#                bestIdx += 1
#            else: 
#                bestIdx -= 1
#        print 'Best cells/metacell for x=%d is %d.' % (x, Ms[bestIdx])
#        bestLs.append(bestIdx)
    return bestVals

def generateMvsLdata():
    'for a constant amount of blocks, how does running time change with (m, l) parameters'
    nCells = 400
    nLeaps = 15
    maxLeap = 300
    nMs = 15
    maxM = int(min(sqrt(nCells) * 2, nCells))
    leap = range(2, maxLeap, int(maxLeap/nMs)+1)
    leap = map(lambda n: n+maxLeap, leap) # make leap 2x bigger
    m = range(1, maxM, int(maxM / nMs)+1)
    
    def myPrint(ln):
        print ln
        os.system('echo "%s" >> mVsL.txt' % ln)
        
    params = {'x': nCells}
    myPrint('# m leap runtime')
    for (m, l) in listCombinations(m, leap):
        params['m'] = m
        params['l'] = l
        try:
            r = getAvgRuntime(WR_CMD % params)
        except AssertionError, e:
            r = 99999
        ln = '%d %d %f' % (m, l, r)
        myPrint(ln)
        
        
def generateCellsVsSpeed():
    maxCells = 10000
    nPts = 20
    cells = map(lambda n: int(n**(log(maxCells)/log(nPts)))+1, range(1, nPts))
    cells = cells[:9] # 12 should correlate to 2080
    params = {}
    useERLeap = True
    ln = '# cells CPU_time'
    #os.system('echo "%s" > cVsS.txt' % ln)
    def myPrint(ln):
        print ln
        os.system('echo "%s" >> cVsS.txt' % ln)
    myPrint(ln)
    for nCell in cells:
        sideLen = int(sqrt(nCell)+1)
        actualCells = sideLen * sideLen
        params['x'] = sideLen
        params['y'] = sideLen
        if useERLeap:
            params['m'] = nCell
            params['l'] = 40 #int(min(sqrt(nCell)*2+1, nCell/2))
        else:
            params['m'] = int(sqrt(nCell))+1
            #params['l'] = int(min(sqrt(nCell)**2+1, nCell/2))
            params['l'] = int(min(sqrt(nCell)*3+1, nCell/2))
        try:
            mu, sd = getRuntimeStats(WR_CMD % params)
            #r = getMinRuntime(WR_CMD % params)
        except AssertionError, e:
            mu, sd = 9999, 9999
        ln = '%d %f %f' % (sideLen*sideLen, mu, sd)
        myPrint(ln)

def generateData():
    maxDups = 15000
    nPts = 20
    dups = map(lambda n: int(n**(log(maxDups)/log(nPts)))+1, range(1, nPts)) # 1 to 1000
    print dups
    bestLs = optimalOverMParam(CMD, dups, getAvgRuntime, min)
    outS = '\n'.join(map(lambda t: str(t[0]) + ' ' + str(t[1]), zip(dups, bestLs)))
    print leapCache
    print outS

if __name__=='__main__':
    #generateData()
    #generateMvsLdata()
    generateCellsVsSpeed()
    #plotData()

    
