/*
 *  bounding_arrays.h
 *  gpu_hierleap
 *
 *  Created by David Orendorff on 2/21/12.
 *  Copyright 2012 UC Irvine. All rights reserved.
 *
 */

#ifndef _BOUNDING_ARRAYS_H_
#define _BOUNDING_ARRAYS_H_

/**
 * Holds arrays that bound state or propensity of an N state vector such 
 * that:
 *	highBound[i] >= highCellBound[i] >= x[i] >= lowCellBound[i] >= lowBound[i]
 */
template <class ValueType>
class BoundingArrays {
public:
	BoundingArrays(unsigned int arraySize) : n(arraySize) {
		highBound = vector<ValueType>(arraySize, NULL);
		highCellBound = vector<ValueType>(arraySize, NULL);
		x = vector<ValueType>(arraySize, NULL);
		lowCellBound = vector<ValueType>(arraySize, NULL);
		lowBound = vector<ValueType>(arraySize, NULL);
	}
	vector<ValueType> highBound, highCellBound, x, lowCellBound, lowBound;
	int size() {return n; }
private:
	const int n;
};

#endif _BOUNDING_ARRAYS_H_