#include "er_leap.h"

#include <vector.h>
#include <queue>
#include <math.h>

#include "../reactions/reaction_set.h"
#include "../reactions/reactions.h"
#include "../state/state.h"
#include "../reactions/reaction_permutation.h"
#include "../reactions/propensity_bounds.h"

using namespace hierleap;

int _erleap_iter = 0;

ReactionPermutation<Reaction*> *ERLeap::ReactionLeap(State *state, Reactions *root){
	assert(root->D(state) > 0);

	// get a list of the upper bounds on rate of all constituent reaction(s)
	int leap = state->numEvents(root);
	vector<Reaction*> & myReactionList = root->descendantReactionList();
	vector<double> upperBoundsD(myReactionList.size());
	
	if (_pb==NULL){
		_pb = new PropensityBoundsNoCells(root, state->X().size(), false);
	}	
	double highestDighDlowRatio = 1.0;

	for (unsigned int i=0; i < myReactionList.size(); i++){
		Reaction *r=  myReactionList[i];
		double upperD =  _pb->upperBoundDrxn(r, state, leap);
		upperBoundsD[i] = upperD;
		double lowBoundD = _pb->lowerBoundDrxn(myReactionList[i], state, leap);
		
		if (lowBoundD==0){
			// find low boundD right before it was zero
			// assume it can proceed with at reaction rate (one of every species), NOTE: this is ok except for dimmerization processes
			lowBoundD = myReactionList[i]->rate();
			assert(lowBoundD <= upperBoundsD[i]);
		}
		
		highestDighDlowRatio = max(highestDighDlowRatio, upperBoundsD[i]/lowBoundD ); // problem: what if div 0?
																						// Prob=0 events can't really happen, but we 
																					// still need a highestDighDlowRatio ratio
	}

	// sample the multinomial according to D^~ normalized by sum
	vector<unsigned int> & rxnEvents = multinomialSelect(upperBoundsD, leap);
	
	// create a reaction permutation
	ReactionPermutation<unsigned int> *rp = new ReactionPermutation<unsigned int>(rxnEvents);
	rp->permute();

	ReactionPermutation<Reaction*> *rpReturn = new ReactionPermutation<Reaction*>(rp, myReactionList);

	// accept with probability :
	//			P = PROD[ F(r) / F^~(r) ]
	// this is REALLY what the upperbound of the whole system must be by definition
	double systemUpperD = sum(&upperBoundsD[0], upperBoundsD.size());
	
	State *stateCpy = new State(state);
	double probAccept = 1.0;
	while (rp->hasNext()){
		int ridx = rp->next();
		Reaction *r = myReactionList[ridx];
		
		if (r->D(stateCpy) > upperBoundsD[ridx]){
			probAccept = probAccept + 1-1;
			assert(false);
		}
		
		
		// P *= f(r) / f^~(r) * D^~ / D
		probAccept *= (r->D(stateCpy) / upperBoundsD[ridx]) * (systemUpperD / root->D(stateCpy)) ; 
		if (probAccept==0 || !isfinite(probAccept)) break;
		//applyReaction(root, stateCpy, r);
		r->fire(stateCpy);
	}
	delete rp;
	delete stateCpy;
	
	// now multiply probAccept by 1/M
	probAccept *= 1 / ipow(highestDighDlowRatio, leap);

	
	if (probAccept > urand()){ 
		_erleap_iter = 0;
		return rpReturn;
	}
	
	_erleap_iter++;
	if (_erleap_iter > 20){
		probAccept += 1 - 1;
	}
	
	delete rpReturn;
	return ReactionLeap(state, root);
}