/*
 * simulator.h
 *
 *  Created on: Sep 3, 2009
 *      Author: Dave Orendorff 
 */

#include <math.h>
#include <string.h>

class Reactions;
class ReactionTauLeaper;
class Historian;
class State;
class TauLeapSSA;
template <class T>
class ReactionPermutation;


#include "simulator.h"

#ifndef FULL_SIMULATOR_H_
#define FULL_SIMULATOR_H_

using namespace hierleap;

// Similar to simulator exceppt it assumes the Sim engine samples 
// 	from time AND reaction events.
class FullSimulator {
public:
	FullSimulator(Reactions *root);
	
	// updates state to reflect SSA equivalent simulation
	//		of reactions locate under *root* for duration *time*
	void run(ReactionTauLeaper *sim, Historian *hist, Reactions *root, double time, int leap, State *state);

	void printRunStats();
	
	// Calculates the probability of reactions occurring over nSamples worth
	//		of reaction leaps. rxnEventsPerSample is the number of steps each
	//		reactionLeap(leap) will at least simulate.
	void calcProbReactions(ReactionTauLeaper *sim, Reactions *root, State *initialState, int leap, int nSamples, int rxnEventsPerSample);
	
	// Applies all of the reactions in the ReactionPermutation to state
	void updateState(State *state, ReactionPermutation<Reaction*> *rxns);

	~FullSimulator() {}

private:
	TauLeapSSA *_ssaSim;
	ReactionTauLeaper *_sim;
	int _totIterations, _rtlEvents, _SSAEvents, _missedHistWrites, ; // For keeping stats on simulation
};

#endif /* SIMULATOR_H_ */
