/*
 *  gpuhier_tau_leap_one_kernel.cpp
 *  gpu_hierleap
 *
 *  Created by David Orendorff on 1/30/11.
 *  Copyright 2011 UC Irvine. All rights reserved.
 *
 */

#include "gpuhier_tau_leap_one_kernel.h"


void GpuhierTauLeapOneKernel::run(State *s, double simTime, int initialLeap, Historian *hist){
	int leap=initialLeap;
	
	double time = 0.0;
	
	//setGpuStateAndTime(s, 0.0);
	setGpuState(s);
	
	_setKernelArgsDependingOnL(leap);
	hist->jot(s, time, 0);
	int err = CL_SUCCESS;
	int iter = -1;
	while (time < simTime){
		iter++;
//		// Too few reaction events to efficiently run GpuHierLeap (?).
//		int minElem = *min_element(s->X().begin(), s->X().end());
//		if (minElem < leap){
//			double dt = 0;
//			while(time <= simTime){// && !hist->jot(s, t, dt)) { // if we want to approach times which we record 'slowly'
//				// also check for D=0
//				ReactionPermutation<Reaction*> *ssaRxns = _ssaSim->ReactionTauLeap(1, s, dt); //_ssaSim
//				time += dt;
//				hist->jot(s, time, dt);
//				delete ssaRxns;
//			}
//			break;
//		}
		
		// _setKernelArgsDependingOnL(leap); todo: run this if L changes.
		
		// set randomUInt index
		cl_int rnOffset = _prepareRandNums(leap*10);
		setRandIdx(rnOffset);
		
		// Set run duration argument.
		float timeToRunSim = hist->nextUpdate() - time;
		setGpuTime(timeToRunSim);
		
		// start kernel.
		err |= clEnqueueNDRangeKernel(_cmd_queue, _hierLeap, 1, NULL, 
									  &_localWorkSize, &_localWorkSize, 0, NULL, NULL);	
		err |= clFinish(_cmd_queue);
		assert(err == CL_SUCCESS);
		
		updateStateFromGpu(s); 
		
		double dt = getSimulatedTime();
		
		bool didWrite = hist->jot(s,time, 0); // This is to bring jot history up to present: kinda hacky!
		if (didWrite) {
			printf("OpenCl kernel ended too late and missed a write to disk, there may be a small bias in results.\n");
		}
		
		time += dt;

		_currentRnIndex = getRandIdx();
		
		// We (implicitly) stopped because we are near the desired 'jot' time.
		if (_currentRnIndex >= 0) {
			dt = 0;
			didWrite = hist->jot(s,time, dt); // to be thorough
			while(!didWrite && time <= simTime) {
				time += dt;
				//double D = _ssaSim->ReactionTauLeap(1, s, dt); 
				double D = 0; assert(false); // note: took ssaSim for compiler purposes.
				didWrite = hist->jot(s,time, dt);
			}
			time += dt; // blaugh
			setGpuState(s);
		}
		
	}
}
