/*
 *  gpuhier_tau_leap_one_kernel.h
 *  gpu_hierleap
 *
 *  Created by David Orendorff on 1/29/11.
 *  Copyright 2011 UC Irvine. All rights reserved.
 *
 */
#include "gpuhier_tau_leap.h"
#include "../utils/gpu_interface.h"

#ifndef GPU_HIER_TAU_ONE_KERNEL_H_
#define GPU_HIER_TAU_ONE_KERNEL_H_

class GpuhierTauLeapOneKernel : public GPUHierLeapSim {
public:

	GpuhierTauLeapOneKernel(State *s, Reactions *root, GpuInterface *gpuInterfaceHierLeap, 
							GpuRandgen *gpuRandgen) : GPUHierLeapSim(s, root, gpuInterfaceHierLeap, gpuRandgen) {

		_gpuOneKernel = new GpuInterface(gpuInterfaceHierLeap->getCmdQueue(), 
										 "/Users/dweo/research/threaded_hierleap/gpusvm/hier_one_kernel.cl");
		_hierLeap = _gpuOneKernel->extractKernel("hierLeap");
		_randIdx_mem = clCreateBuffer(gpuInterfaceHierLeap->getContext(), CL_MEM_READ_WRITE, sizeof(cl_int), NULL, NULL);
		_setKernelArgs();
		printf("Running one hierleap kernel.\n");
		
		int preferMultipleOf = 32;
		int bestIfUnlimitedWgSize = _nCells <= preferMultipleOf ? _nCells : floor((float)_nCells / preferMultipleOf) * preferMultipleOf;
		_localWorkSize = min(gpuInterfaceHierLeap->maxWorkgroupSize(), bestIfUnlimitedWgSize);
		printf("GpuHierLeap workgroup size %d\n", _localWorkSize);
	}
	
	virtual ~GpuhierTauLeapOneKernel() {
		printf("In ~GpuhierTauLeapOneKernel destructor.\n");
		cl_int err = clReleaseMemObject(_randIdx_mem);
		err |= clReleaseKernel(_hierLeap);
		assert(err == CL_SUCCESS);
		delete _gpuOneKernel;
	}
	
	virtual void run(State *s, double simTime, int initialLeap, Historian *hist);	
	
	// Returns the value stored in global memory for totalSimtime
	float getSimulatedTime() {
		float t;
		cl_int err = clEnqueueReadBuffer(_cmd_queue, _time_mem, CL_TRUE, 0, sizeof(cl_float),
										 (void*)&t, 0, NULL, NULL);
		err |= clFlush(_cmd_queue);
		assert(err == CL_SUCCESS);
		return t;
	}
	
	// Stores this value in _randIdx_mem.
	void setRandIdx(cl_int rndIdx) {
		assert(rndIdx >= 0);
		cl_int err = clEnqueueWriteBuffer(_cmd_queue, _randIdx_mem, CL_TRUE, 0, sizeof(cl_int),
								   (void*)&(rndIdx), 0, NULL, NULL);
		clFinish(_cmd_queue);
		assert(err == CL_SUCCESS);
	}
	
	// Returns the value stored in _randIdx_mem
	int getRandIdx() {
		int i;
		cl_int err = clEnqueueReadBuffer(_cmd_queue, _randIdx_mem, CL_TRUE, 0, sizeof(cl_int),
										 (void*)&i, 0, NULL, NULL);
		err |= clFlush(_cmd_queue);
		assert(err == CL_SUCCESS);
		return i;
	}
	
private:
	int _leapArgIdx, _orderedRxnEventsIdx, _DIdx;
	float _simTimeArgIdx;
	size_t _localWorkSize;
	
	void _setKernelArgs() {
		int totRxnSets = 1 + _nCells + _nBaseRxn;
		
		int argIdx = 0;

		cl_int err = clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_state_mem);
		assert(err == CL_SUCCESS);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(int), &_nSpecies);
		_simTimeArgIdx = argIdx;
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_time_mem);
		assert(err == CL_SUCCESS);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_cellStartIdx_mem);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_rates_mem);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_inpSpec_mem);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_outSpec_mem);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_highDeltaS_mem);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_lowDeltaS_mem);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_highDeltaCell_mem);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int), &_nHighDeltaCell);
		assert(err == CL_SUCCESS);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_lowDeltaCell_mem);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int), &_nLowDeltaCell);
		assert(err == CL_SUCCESS);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_randomUint_mem); 
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_mem), &_randIdx_mem); 
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int), &_totalRandNums); 
		// leap
		_leapArgIdx = argIdx;
		argIdx++;
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * _nSpecies, NULL); // highBoundS
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * _nSpecies, NULL); 
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * _nSpecies, NULL); 
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * _nSpecies, NULL); 
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * _nSpecies, NULL); 
		assert(err == CL_SUCCESS);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_float) * totRxnSets, NULL); // DboundHigh
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_float) * totRxnSets, NULL);
		_DIdx = argIdx;
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_float) * totRxnSets, NULL); // fyi, also needs to be of size at least |leap|
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_float) * totRxnSets, NULL);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_float) * totRxnSets, NULL);
		assert(err == CL_SUCCESS);
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * totRxnSets, NULL); // rxnEvents
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * _nSpecies, NULL); // prevState
		_orderedRxnEventsIdx = argIdx;
		argIdx++;
		argIdx++; // taus
		
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * (_nCells + 1), NULL); // cellStartIdx
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_float) * _nBaseRxn, NULL); // rates
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int2) * _nBaseRxn, NULL); // inpSpec
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int2) * _nBaseRxn, NULL); // outSpec
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * _nSpecies, NULL); // highDeltaS
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(cl_int) * _nSpecies, NULL);
		
		
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(struct DeltaX) * _nHighDeltaCell, NULL); // scDeltaHighCell
		err |= clSetKernelArg(_hierLeap, argIdx++, sizeof(struct DeltaX) * _nLowDeltaCell, NULL); 
		
		assert(err == CL_SUCCESS);
	}
	
	void _setKernelArgsDependingOnL(int leap) {
		int totRxnSets = 1 + _nCells + _nBaseRxn;
		
		cl_int err = clSetKernelArg(_hierLeap, _leapArgIdx, sizeof(cl_int), &leap);
		err |= clSetKernelArg(_hierLeap, _orderedRxnEventsIdx, sizeof(cl_int) * leap, NULL);
		err |= clSetKernelArg(_hierLeap, _orderedRxnEventsIdx+1, sizeof(cl_float) * leap, NULL); // taus
		
		if (leap > totRxnSets) {
			err |= clSetKernelArg(_hierLeap, _DIdx, sizeof(cl_float) * leap, NULL); // taus
		}
		
		assert(err == CL_SUCCESS);
	}
	
	cl_kernel _hierLeap;
	
	cl_mem _randIdx_mem;
	
	GpuInterface *_gpuOneKernel;
};

#endif