/*
 * hier_leap.h
 *
 *  Created on: Sep 3, 2009
 *      Author: David Orendorff
 */

#ifndef HIER_LEAP_H_
#define HIER_LEAP_H_

#include "reaction_leaper.h"

template <class T> class ReactionPermutation;
class State;
class Reactions;
class PropensityBounds;
namespace hierleap{
	class Reaction;
}
using namespace hierleap;
// Samples L reaction events according to the Hiearcichal ER-Leap algorithm
//		proposed by Mjolsness, Orendorff (V4)
class HierLeap : public ReactionLeaper {
public:

	HierLeap(State *s, Reactions *root);
	
	// Returns an ordered list of reactions of length leap :
	//	- sampled from SSA equivalent process
	//	- this is guaranteed to be recursive with bottom call to ERLeap or SSA
	//	- state will NOT be changed
	ReactionPermutation<Reaction*> *ReactionLeap(State *state, Reactions *root);

	// Calculate probability of acceptance as :
	//		Psig = PROD[ F(r) / F(r)^bar * D(children with r)^bar / D(children with r) ]
	double acceptanceProbability(State *initial, State *assumeChildFullDiffusion,
			ReactionPermutation<Reaction*> * fullOrdering, Reactions *root);

	// Isolates the direct children of root by assuming that all blocks diffuse as much as they can
	//		into neighboring blocks (contained in root).
	State *diffuseChildrenTogether(Reactions *root, State* state);
	
	// Given the ordering in metaReactions (which leaves fire) and the order the children fired,
	//		create a new RP by permuting when children fire relative to each other, but the same to self.
	ReactionPermutation<Reaction*> *sigmaOrdering(ReactionPermutation<int> *metaRxns, 
			vector<ReactionPermutation<Reaction*>*> & childrenRps){
		ReactionPermutation<Reaction*> *fullOrder = new ReactionPermutation<Reaction*>();
		
		//for (int i=0; i < metaRxns.size(); i++){
		while(metaRxns->hasNext()){
			int cellIdx = metaRxns->next();
			Reaction *r = childrenRps[cellIdx]->next();
			fullOrder->addBack(r);
		}
		return fullOrder; // TODO may need to delete stuff
	}

	vector<double> & upperBoundChildrenD(Reactions* root, State *state);

private:
	PropensityBounds *_bounder;

};

#endif /* HIER_LEAP_H_ */
