/*
 *  hierleap_calculations.cpp
 *  gpu_hierleap
 *
 *  Created by David Orendorff on 1/10/11.
 *  Copyright 2011 UC Irvine. All rights reserved.
 *
 */

#include "hierleap_calculations.h"

#include <math.h>
#include <boost/foreach.hpp>
#include <assert.h>

float HierleapCalculations::computePCoarse(vector<float> DboundHigh, vector<float> DboundCellHigh, vector<float> DboundLow, 
					 vector<float> DboundCellLow, vector<int> rxnEvents, vector<float> taus) {
	
	//  take product of cells i, (DhighCell / Dhigh) ^ rxnEvents[i]
	double pCoarse = 1.0;
	for (int i = 1; i <= _nCells; i++){
		if (!rxnEvents[i]) continue;
		
		pCoarse *= pow(DboundCellHigh[i] / DboundHigh[i], rxnEvents[i]);
		assert(pCoarse >= 0 && pCoarse <= 1.0);
	}
	
	double tauSum = 0.0;
	BOOST_FOREACH(float x, taus) tauSum += x;
	
	pCoarse = pCoarse * exp(-tauSum * (DboundCellLow[0] - DboundLow[0]));
	//pCoarse = exp(-tauSum * (DboundCellLow[0] - DboundLow[0]));
	assert(pCoarse >= 0 && pCoarse <= 1.0);
	return (float) pCoarse;
}