#include "historian.h"


#include <fstream.h>
#include <vector.h>
#include <string.h>
#include <sstream>
#include <limits>
#include <boost/foreach.hpp>
#include <sys/resource.h>

#include "../globals.h"
#include "../state/state.h"
#include "../state/species.h"
#include "../reactions/reactions.h"

using namespace std;

Historian::Historian(State *state, string outDir, double totalSimTime, double updatePeriod):
	_updatePeriod(updatePeriod), _nextUpdate(0.0), _totalSimTime(totalSimTime) {
	
	_initialState = state->X(); // should copy automatically (?)
	_prevState = _initialState;
	
	if (outDir.size() > 0 && outDir[outDir.size()-1] != OS_SEP){
		outDir += OS_SEP;
	}
	
	// make sure we can open this many files
	//	RLIMIT_NOFILE 
	struct rlimit *rlp = new struct rlimit();
	bool ret = getrlimit(RLIMIT_NOFILE, rlp);
	if (rlp->rlim_cur <= state->species().size() + 3) {
		printf("By default, can only have %x=%d files open at a time, changing value to maximum allowed (%x=%d), size rlim %d.\n", 
			   rlp->rlim_cur, rlp->rlim_cur, rlp->rlim_max, rlp->rlim_max, sizeof(rlp->rlim_cur));
		rlp->rlim_cur = 1257; //rlp->rlim_max;
		ret |= setrlimit(RLIMIT_NOFILE, rlp);
	}
	assert(ret == 0);
		
	// setup files
	vector<Species*> & allSpecies = state->species();
	_speciesFiles.resize(allSpecies.size());
	for (int i=0; i < allSpecies.size(); i++){
		
		// get x,y,z of cell species is in
		int x,y,z;
		allSpecies[i]->getParent()->getCoordinate(x,y,z);
		
		stringstream filename;
		filename << outDir << allSpecies[i]->name() << '_' << allSpecies[i]->getUniqueId()
					<< '-' << x <<'-'<<y<<'-'<<z<<".txt";
		ofstream *f = new ofstream();
		f->open(filename.str().c_str(), ofstream::in | ofstream::trunc);
		if (!f->good()) {
			system("pwd");
			printf("Unable to open file %s.\n", filename.str().c_str());
			assert(false);
		}
				
		_speciesFiles[i] = f;
	}
	_timeFile = new ofstream();
	_timeFile->open(string(outDir + "time.txt").c_str());
	assert(_timeFile->good());
	
	
}

void Historian::nextRun(){
	_prevState = _initialState;
	BOOST_FOREACH(ofstream *f, _speciesFiles){
		*f << "\n";
	}
	*_timeFile << "\n";
	_nextUpdate = 0.0;
}


void Historian::_jot(State *s, double time, double dt){
	
	for (int i=0; i < _prevState.size(); i++){
		(*_speciesFiles[i]) << _prevState[i] << " ";
	}
	*_timeFile << time << " ";
	
	_nextUpdate += _updatePeriod;
	if (_nextUpdate > _totalSimTime)
		_nextUpdate = std::numeric_limits<double>::infinity();;

	jot(s, time, dt); // make sure we don't need to do it all again
	_rememberState(s); // rember prev state *after* we are done jotting.
}
