#ifndef HISTORIAN_H_
#define HISTORIAN_H_

#include <fstream.h>
#include <vector.h>
#include <string.h>
#include <assert.h>
#include <boost/foreach.hpp>

#include "../state/state.h"

using namespace std;

class State;

class Historian {
public:
	
	Historian(){
		_totalSimTime = 0;
		_timeFile = NULL;
	}
	
	Historian(State *s, string outDir, double totalSimTime, double updatePeriod);
	
	virtual bool jot(State *state, double time, double delta_time){
		assert(_timeFile); // make sure default constructor wasn't called.
		if ( _nextUpdate > _totalSimTime || (time+delta_time) < _nextUpdate){
			_rememberState(state);
			return false;
		}
		_jot(state, time, delta_time); 
		return true;
	}
	
	 // a run is over and a new one is about to start
	virtual void nextRun();
	
	virtual ~Historian(){
		// close and delete ofstreams
		BOOST_FOREACH(ofstream *f, _speciesFiles){
			f->close();
			delete f;
		}
		if (_timeFile) {
			_timeFile->close(); 
			delete _timeFile;
		}
	}
	
	double nextUpdate(){
		return _nextUpdate;
	}
	
protected:
	double _updatePeriod, _nextUpdate, _totalSimTime;
	
private:
	void _jot(State *s, double time, double dt);
	
	void _rememberState(State *s){
		_prevState = s->X(); // this should copy (right ?)
	}
	

	vector<int> _prevState;
	vector<int> _initialState; // Upon initialization, we remember the value of state->X
	ofstream *_timeFile; // records when the 'jots' occur
	vector<ofstream*> _speciesFiles; // index in each corresponds to index of species in state
};

/*
 * The slacker historian that does not really write to file.
 */ 
class DumbHistorian : public Historian {
public:
	DumbHistorian(double totalSimTime) {
		_updatePeriod = totalSimTime;
		_nextUpdate = 0.0;
		_totalSimTime = totalSimTime;
	}
	
	virtual bool jot(State *state, double time, double delta_time){
		if ( _nextUpdate > _totalSimTime || (time+delta_time) < _nextUpdate){
			return false;
		}
		_nextUpdate += _updatePeriod;
		if (_nextUpdate > _totalSimTime)
			_nextUpdate = std::numeric_limits<double>::infinity();
		printf("%f\n", time);
		jot(state, time, delta_time);
		return true ;
	}
	
	// a run is over and a new one is about to start
	virtual void nextRun() {
		_nextUpdate = 0.0;
	}
	
private:
	//double _nextUpdate;
	//const double _updatePeriod, _totalSimTime;
};

#endif /*HISTORIAN_H_*/
