#include "simulator.h"

#include "../reactions/propensity_bounds.h"

using namespace hierleap;

Simulator::Simulator(Reactions *root, State *state){
	// TODO assumes we are in a cell configuration
	_pb = new PropensityBounds(root, state->speciesTypes(), true); 
}

//~Simulator::Simulator(){
//	delete _pb;
//}

// Samples delta time (tau) in the standerd SSA way for all L reactions
double Simulator::_tauSampleSSA(Reactions *root, State *initialState, ReactionPermutation<Reaction*> *orderedReactions){
	assert(orderedReactions);
	double tauSum = 0;
	State *s = new State(initialState);
	ReactionPermutation<Reaction*>  *rpCopy = new ReactionPermutation<Reaction*> (orderedReactions);
	rpCopy->lockOrder();
	
	while (rpCopy->hasNext()){
		double tau = erand() * 1 / root->D(s);
		tauSum += tau;
		Reaction *next = rpCopy->next();
		next->fire(s);	//  make sure you do prob accept, and THEN fire!
		
	}
	delete s;
	delete rpCopy;
	return tauSum;
}

// Given the ordered list of reactions, samples a dT equivalent to SSA
double Simulator::_tauSample(Reactions *root, State *initialState, ReactionPermutation<Reaction*> *orderedReactions){
	assert(initialState->numEvents(root) > 0);
	
	int leap = initialState->numEvents(root) ;
	
	double lowD = _pb->lowerBoundDrxns(root, initialState, leap); // pb: propensity bounds
	double highD = _pb->upperBoundDrxns(root, initialState, leap);
	
	// TODO if probabity of early accept is probably too low, just do simple SSA tau update
	
	double tauSum = erlang_sample(lowD, leap); // TODO confirm math
	double tauProbEarly = exp(-tauSum * (highD - lowD));
	
	
	if (urand() <= tauProbEarly) {
		return tauSum;
	}	
	
	// try for late acceptance
	// sample individual taus...
	State *stateCpy = new State(initialState);
	double tsum = 0.0;

	vector<double> taus;
	for (int i=0; i < leap; i++){
		double rv = erand();
		assert(rv);
		taus.push_back(rv);
		tsum += rv;
	}

	// normalize taus, but not until very end!
	float normalize_factor = tauSum / tsum;

	float expSum = 0;
	ReactionPermutation<Reaction*>  *rpCopy = new ReactionPermutation<Reaction*> (orderedReactions);
	rpCopy->lockOrder();

	
	int i=0;
	while (rpCopy->hasNext()){
		Reaction *next = rpCopy->next();
		
		if (lowD > root->D(stateCpy) + lowD / 1000){
			assert(false);
		}
		if (highD < root->D(stateCpy) - highD / 1000){
			assert(false);
		}
		
		expSum += taus[i++] * (root->D(stateCpy) - lowD);
		
		next->fire(stateCpy);	//  make sure you do prob accept, and THEN fire!
	}
	
	delete rpCopy;
	delete stateCpy;
	
	double P =  exp(-expSum * normalize_factor);
	double Q = (P - tauProbEarly) / (1 - tauProbEarly);
	assert(0 <=Q && Q <= 1);
	
	
	if (P < 0.1){
		// TODO there is probably a problem here
		int z = 1;
		z++;
		P += 1 - 1;
	}
	
	if (urand() <= Q){
		return tauSum;
	}
	
	// resample, tail recursive for efficiency
	return _tauSample(root, initialState, orderedReactions);
}