/*
 * SSA.cpp
 *
 *  Created on: Sep 9, 2009
 *      Author: dweo
 */

#include "ssa.h"

#include "../state/state.h"

using namespace hierleap;

// Use Gillespies SSA algorithm (reaction component) to choose L reaction
//		events to fire. Return a reaction permutation of result.
// XXX: not very efficient if we are only stepping a small number of steps at a time
ReactionPermutation<Reaction*> *SSA::ReactionLeap(State *state, Reactions *root){
	State *stateCpy = new State(state);
	int steps = stateCpy->numEvents(root);
	vector<Reaction*> & reactions = root->descendantReactionList();
	ReactionPermutation<Reaction*> *rp = new ReactionPermutation<Reaction*>();

	for (int i=0; i < steps; i++){

		// choose which reaction to fire
		double allD = root->D(stateCpy);
		double targetSum = urand() * allD;
		int ridx = 0;
		double sum=0;
		

		for (  ;; ridx++) {
			
			sum += reactions[ridx]->D(stateCpy);
			if (sum >= targetSum) break;
		}

		assert(ridx < reactions.size());
		Reaction *r = reactions[ridx];

		// add r to reaction permutation, fire and adjust stateCpy accordingly
		rp->addBack(r);
		r->fire(stateCpy);
	}

	rp->lockOrder();
	delete stateCpy;
	return rp;
}
