/*
 *  tau_leap_ssa.cpp
 *  gpu_hierleap
 *
 *  Created by David Orendorff on 12/22/11.
 *  Copyright 2011 UC Irvine. All rights reserved.
 *
 */

#include "tau_leap_ssa.h"

#include "reaction_tau_leaper.h"
#include "../reactions/reactions.h"
#include "../reactions/reaction.h"
#include "../state/state.h"
#include "../reactions/reaction_set_map.h"
#include "../reactions/delta_propensity_bounds.h"
	
void TauLeapSSA::changeRuntimeParameters(State *s, int leap) {
	_propensity = _root->DbyUniqueId(s->X());
}

double TauLeapSSA::ReactionTauLeap(int leap, State *s, double &dt){
	assert(_propensity >= 0);
	assert(leap > 0);
	
	vector<Reaction*> & reactions = _root->descendantReactionList();
	dt = 0;
	
	// choose which reaction to fire
	while (leap--) {
		if (isNearlyEqual(_propensity, 0)){
			dt = INFINITY;
			return _propensity;
		}
		
		double targetSum = urand() * _propensity;
		int ridx = 0;
		
		for (double sum=0  ;; ridx++) {
			sum += reactions[ridx]->D(s);
			if (sum >= targetSum) break;
		}
		
		assert(ridx < reactions.size());
		Reaction *r = reactions[ridx];
		
		_propensity += _getDeltaComputer(reactions[ridx])->eval(s->X());
		r->fire(s);
		assertNearlyEqual(_propensity, _root->D(s), 1e-10); // todo: for debuggin only.
		
		dt += erand() / _propensity;
	}
	return _propensity; 
}
	

void TauLeapSSA::_initDeltaEquations() {
	assert(_root);
	_deltaCompute = new ReactionSetMap<DeltaEquation*>(_root->numRxnSets());
	BOOST_FOREACH(Reaction *r, _root->descendantReactionList()) {
		(*_deltaCompute)[r] = new DeltaEquation(_root, r);
	}
}
	
DeltaEquation* TauLeapSSA::_getDeltaComputer(Reaction *r) {
	return checkNotNull((*_deltaCompute)[r]);
}
	