#ifndef TAU_LEAP_SSA_H_
#define TAU_LEAP_SSA_H_

#include "reaction_tau_leaper.h"

class State;
class Reactions;

class DeltaEquation;

template <class DE>
class ReactionSetMap;

/*
 * Simple SSA with deltaTime (tau) and new state computed.
 * Can only be run with leap=1 (although easy to generalize)
 */
class TauLeapSSA : public ReactionTauLeaper {
	
public:
		
	TauLeapSSA(Reactions *root, State *s):_root(root), _propensity(-1) {
		_initDeltaEquations();
	}
	
	TauLeapSSA(Reactions *root):_root(root), _propensity(-1){ 
		_initDeltaEquations(); 
	}
	
	virtual char* name() { return "TauLeapSSA";}
	
	virtual void changeRuntimeParameters(State *s, int leap);
	
	virtual double ReactionTauLeap(int leap, State *s, double &dt);

	~TauLeapSSA() {
		delete _deltaCompute;
	}
private:
	double _propensity;
	Reactions *_root;
	ReactionSetMap<DeltaEquation*> * _deltaCompute;
	
	void _initDeltaEquations();
	
	DeltaEquation* _getDeltaComputer(Reaction *r);
	
};

#endif