/*
 *  threshold_optimal_leap.h
 *  gpu_hierleap
 *
 *  Created by David Orendorff on 1/13/12.
 *  Copyright 2012 UC Irvine. All rights reserved.
 *
 */
#ifndef THREASHOLD_OPTIMAL_LEAP_H_
#define THREASHOLD_OPTIMAL_LEAP_H_

#include "../reactions/reactions.h"
#include "state.h"
#include "reaction_tau_leaper.h"

#define HUGE_LEAP 9999

/*
 * For a given root and state, what is the highest possible value of L that 
 * results in an acceptance probabilty of at least thresholdAcceptanceProbability 90% 
 * of the time?
 */
class ThresholdOptimalLeap {
public:
	ThresholdOptimalLeap(ReactionTauLeaper *alg, Reactions *root, State *state) : _alg(alg), _root(root), _state(state) {
		_origState = vector<int>(_state->X().begin(), _state->X().end());
	}
	
	double optimalLeap(double thresholdAcceptanceProbability) {
		double thresh = thresholdAcceptanceProbability;
		
		// The one variable we change is L
		double pAccept =0.7;
		int stepSize = 5;
		int leap = stepSize;
		
		// first, get into the correct range of values...
		while (pAccept > thresholdAcceptanceProbability && leap < HUGE_LEAP) {
		//for (; pAccept > thresholdAcceptanceProbability; leap += stepSize) {
			assert(pAccept > thresholdAcceptanceProbability);
			printf("pAcccept(%f)\n", pAccept);
			pAccept = _acceptProb(leap);
			
			if (pAccept > 1 - (1.0-thresh)/2.0) {
				stepSize++;
				stepSize *= 1.2;
			} else if (pAccept < 1 - (1.0-thresh)/1.1) {
				stepSize = leap / 100 + 1;
			}
			leap += stepSize;
			leap = min(HUGE_LEAP, leap);
		}
		
		// random walk on leap, based on thresh criterion
		int samples = 500;
		leap -= stepSize + 1;
		while (_acceptProb(++leap) >= thresholdAcceptanceProbability && leap < HUGE_LEAP);
		
		double leapSumTooHigh = 0.0;
		int nPtooHigh = 0;
		for (int i = 0; i < samples; i++) {
			if (_acceptProb(leap) >= thresholdAcceptanceProbability) {
				leapSumTooHigh += leap;
				nPtooHigh++;
				leap++;
			} else {
				leap--;
				assert(leap >= 1);
			}
		}
		
		// sort of a hacky way to report this number, doh.
		return leapSumTooHigh / nPtooHigh;
	}
	
private:
	Reactions *_root;
	State *_state;
	ReactionTauLeaper *_alg;
	vector<int> _origState;
	
	// sample 
	double _acceptProb(int leap) {
		assert(leap > 1);
		_alg->changeRuntimeParameters(_state, leap);
		
		//ReactionTauLeap(int leap, State *s, double &dt)
		double elapsed = 0;
		double dSum = _alg->ReactionTauLeap(leap, _state, elapsed);
		assert(dSum > 0);
		
		_state->setXdirectly(_origState);
		double p = _alg->getPreviousAcceptanceProbability();
		assert(p >= 0.0 && p <= 1.0);
		return p;
	} 
};

#endif