/*
 * reaction_cache.h
 *
 *  Created on: Sep 25, 2009
 *      Author: dweo
 */

#ifndef REACTIONCACHE_H_
#define REACTIONCACHE_H_

using namespace std;

#include <vector>
#include <queue>
#include <assert.h>

// Holds information regarding every ReactionSet's state
// Note: (probably) no computation will be done here, just information will be held
class ReactionCache {
public:
	ReactionCache(int speciesTypes, int numDependents) : _DhighCell(-1), _Dhigh(-1) { 
		assert(speciesTypes>=0); assert(numDependents>=0);
		_mstb.resize(speciesTypes);
		_upperDchildren.resize(numDependents);
		_rxnEvents=0;
	}

	// Maximal species by type
	vector<int> & msbt(){ return _mstb; }
	void setNumEvents(int e){ _rxnEvents = e; }
	int numEvents(){ return _rxnEvents; }
	
	// For all children reactions
	vector<double> & upperDchildren(){
		return _upperDchildren; 
	}
	
	void setDhighCell(double D){assert(D>= 0); _DhighCell=D;}
	double DhighCell(){
		assert(_DhighCell >= 0); 
		return _DhighCell;
	}
	
	void setDhigh(double D){assert(D >= 0); _Dhigh = D;}
	double Dhigh(){assert(_Dhigh >= 0); return _Dhigh;}
	
	
private:
	vector<int> _mstb; 
	double _tau;
	double _DhighCell;
	double _Dhigh;
	vector<double> _upperDchildren; // holds the upper bounds on D for children
	int _rxnEvents;
	
};

#endif /* REACTIONCACHE_H_ */
