/*
 * Species.cpp
 *
 *  Created on: Sep 11, 2009
 *      Author: dweo
 */

#include "species.h"

#include <boost/foreach.hpp>

#include "../reactions/reaction_set.h"
#include "../reactions/reactions.h"
#include "../reactions/reaction.h"

using namespace hierleap;

int Species::_idNumberCounter = 0;

// A change in this species results in these reaction propensities changing.
vector<Reaction*> Species::changesPropensityFor() {
	if (_changesPropensityFor == NULL) {
		_changesPropensityFor = new vector<Reaction*>();
		assert(_parent);
		Reactions * par = getParent();
		Reactions * root = par->getRoot();
		BOOST_FOREACH(Reaction *r, par->getRoot()->descendantReactionList()) {
			switch (r->numReactants()) {
				case 0:
					continue;
				case 2:
					if (r->getReactantId(1) == getUniqueId())
						break;
				case 1:
					if (r->getReactantId(0) == getUniqueId())
						break;
					continue;
				default:
					assert(false); // assume at most two reactants for now.
			} 
			
			// if we're here then we know s changes propensity for reaction.
			_changesPropensityFor->push_back(r);
		}
	}
	return vector<Reaction*>(_changesPropensityFor->begin(), _changesPropensityFor->end());
}

Reactions* Species::getParent(){ 
	assert(_parent==NULL || !_parent->isReaction());
	return _parent;
}

void Species::setParent(Reactions* r){
	if (r) assert(!r->isReaction());
	_parent = r;
}

//int Species::_deltaSpecies(Reaction *r){
//	// look at each product, and see if is *this*
//	int delta = 0; 
//	for (int i=0; i < r->numReactants(); i++){
//		if (r->getReactantId(i) == _uniqueId){
//			delta -= r->getReactantStoic(i);
//		}
//	}
//	for (int i=0; i < r->numProducts(); i++){
//		if (r->getProductId(i) == _uniqueId){
//			delta += r->getProductStoic(i);
//		}
//	}
//	return delta;
//}

int Species::_maxDelta(Reactions *connectingGroup, int funcId){
	int dMax = 0;
	int multiplier = funcId == INCREASE_FUNC ? 1 : -1;

	BOOST_FOREACH(ReactionSet *r, connectingGroup->descendantReactionList()){
		int d = multiplier * r->asReaction()->delta(this->getUniqueId()); //_deltaSpecies(r->asReaction());
		dMax = max(dMax, d);
		assert(dMax >= 0);
	}
	return dMax;
}

vector<ConnectingRxns*> Species::_buildConnectingGroups(int FUNC_ID){
	vector<ConnectingRxns*> arr;
	vector<ReactionSet*> higherUps(_parent->getNeighbors().begin(), _parent->getNeighbors().end());
	higherUps.push_back(_parent);
	BOOST_FOREACH(ReactionSet *r, higherUps){
		int v = _maxDelta(r->asReactions(), FUNC_ID);
		if (v > 0){
			ConnectingRxns *c = new ConnectingRxns(r->asReactions(), v);
			arr.push_back(c);
		}
		assert(v >= 0);
	}
	return arr;
}