/*
 * species.h
 *
 *  Created on: Sep 11, 2009
 *      Author: dweo
 */

#ifndef SPECIES_H_
#define SPECIES_H_

#include <assert.h>

#include <string>
#include <vector>

#include "../utils/common.h"

#define INCREASE_FUNC 1
#define DECREASE_FUNC 2

using namespace std;
class Reactions;

namespace hierleap{
class Reaction; 

// Species contains a list of these structs
class ConnectingRxns {
public:
	ConnectingRxns(Reactions *r, int delta){
		_parentConnecting = r; _maxDeltaXPerEvent = delta;
	}
	
	// compare rxnId, if those are equal, compare deltas. 
	bool operator<(ConnectingRxns c) {
		return _parentConnecting != c._parentConnecting ? 
		_parentConnecting < c._parentConnecting : 
		abs(_maxDeltaXPerEvent) < abs(c._maxDeltaXPerEvent);
	}
	
	bool operator==(ConnectingRxns c) {
		return _parentConnecting == c._parentConnecting;
	}
	
	Reactions *_parentConnecting;
	int _maxDeltaXPerEvent;
};

class Species {
public:
	Species(): _uniqueId(-1), _changesPropensityFor(NULL), _parent(NULL){}
	
	Species(Species *s): _uniqueId(-1), _changesPropensityFor(NULL), _parent(NULL) {
		_type = s->getTypeId();
		_name = string(s->name());
	}
	
	Species(char *name) : _uniqueId(-1), _changesPropensityFor(NULL), _parent(NULL){
		_name = string(name);
	}
	
	Species(string name) : _uniqueId(-1), _changesPropensityFor(NULL), _parent(NULL){
		_name = name;
	}
	
	Species(char *name, int x): _uniqueId(-1), _changesPropensityFor(NULL), _parent(NULL){ assert(false); }// more work todo here

	// kinda dumb, need this to sort species.
	bool operator<(Species & s) {
		return getUniqueId() < s.getUniqueId();
	}
	
	// Gives species a unique id number, this function can only be called once/species
	void registerMe(){
		assert(_uniqueId < 0); // not registered yet
		_uniqueId = _idNumberCounter++;
	}

	// assert this functions are only called once
	void setTypeId(int type){ _type = type; }
	int getTypeId(){ return _type;}

	void setUniqueId(int id){ _uniqueId = id; }
	int getUniqueId(){  return _uniqueId;}

	void setParent(Reactions* r);
	
	// The parent of a species is defined as the parent of some rxn that uses *this* as a reactant.
	Reactions* getParent();

	const char* name(){ return _name.c_str(); }

	// returns how much the reaction r increases/decreases this species
	// int _deltaSpecies(Reaction *r);
	
	// Not the best way to do this, but not complicated at least
//	int _funCall(int funcId, Reaction *r){
//		switch(funcId){
//		case INCREASE_FUNC: return _increasesSpecies(r); break;
//		case DECREASE_FUNC: return _decreasesSpecies(r); break;
//		}
//		assert(false); return -1;
//	}
	
	int _maxDelta(Reactions *connectingGroup, int funcId);
	
	vector<ConnectingRxns*> _buildConnectingGroups(int FUNC_ID);
	
	// Which parent reaction sets *may* increase the *reactants* of this species if they are fired. 
	vector<ConnectingRxns*> & increasingConnectingGroups(){
		assert(false); // obsolete, use ConnectionGrandmaster
		if (_increasingGroups.size() > 0){ return _increasingGroups; }
		return _increasingGroups = _buildConnectingGroups(INCREASE_FUNC);
	}
	
	vector<ConnectingRxns*> & decreasingConnectingGroups(){
		assert(false); // obsolete, use ConnectionGrandmaster
		if (_decreasingGroups.size() > 0){ return _decreasingGroups; }
		return _decreasingGroups = _buildConnectingGroups(DECREASE_FUNC);
	}
	
	// A change in this species results in these reaction propensities changing.
	vector<Reaction*> changesPropensityFor();
	
protected:

	static int _idNumberCounter;

	int _type;
	int _uniqueId;
	string _name;

	Reactions *_parent; 
	
	// how much each connecting group (1 level parents) may change this species
	vector<ConnectingRxns*> _increasingGroups; 
	vector<ConnectingRxns*> _decreasingGroups;
	vector<Reaction*> *_changesPropensityFor;
};
}

#endif /* SPECIES_H_ */
