/*
 * State.h
 *
 *  Created on: Sep 7, 2009
 *      Author: dweo
 */

#include <queue>

using namespace std;

#ifndef STATE_H_
#define STATE_H_

//class State;
class ReactionCache;
class ReactionSet;
class Reactions;

namespace hierleap{
	class Reaction;
	class Species;
}
using namespace hierleap;


// State contains the following information regarding molecule count and any cache
//		- molecule count for species
//		- cache for reactions to store (ie, for computation of D).
class State {
public:
	State();

	// TODO Maybe do a lazy copy (?)
	State(const State *s);

	State(vector<Species*> species, Reactions* root);

	// initialXbySpecies should be a vector of size |species types|
	void setXbySpeciesId(vector<int> & initialXbySpecies);
	
	// directly setup x, note this isn't too handy if you have multiple species types
	void setXdirectly(vector<int> x);

	inline int x(int speciesId){ return _x[speciesId]; }
	int x(Species *s);

	void changeX(int speciesId, int deltaX);
	
	inline void changeX(Species *s, int deltaX);
	
	// The maximum of each species of all children for every TYPE is returned
	vector<int> & maximalSpeciesByType(ReactionSet *r);

	vector<int> & cacheMSBT(ReactionSet *r);

	// Sets all numEvents for all reactions to 0
	void clearEvents();
	
	int numEvents(ReactionSet *r);

	void setEvents(ReactionSet *r, int events);

	// Provides the sufficient statistics for computing D-bounds
	//		- it may be a list of species numbers by species TYPE
	//			- always returns a vector of length ||Species Types||
	vector<int> & gridcellSpecies(Reactions *root);
	
	int speciesTypes(){ return _numSpeciesTypes; } 

	vector<int> & X(){ return _x; }
	
	int speciesSize() { return X().size(); }
	
	double DtildeCached(ReactionSet *r);
	
	Species* species(int idx){ return _species[idx];}
	
	vector<Reaction*> & touchingRxn(int speciesId);
	
	vector<Species*> & species(){
		return _species;
	}
	
	ReactionCache* cache(ReactionSet *r);
	
	virtual ~State();
private:
	vector< vector<Reaction*> *> _touchingRxn;
	void _setupReactionCache(ReactionSet *root);

	vector<int> & _maximalSpeciesByType(Reactions *r);
	vector<int> & _maximalSpeciesByType(Reaction *r);

	int _numSpeciesTypes;
	vector<int> _x;
	vector<ReactionCache*> _reactionCache; // store misc information pertinent to all reaction(s)
	Reactions *_root;
	vector<Species*> _species; // for reference only, no modification
};



#endif /* STATE_H_ */
