/*
 *  gpu_randgen_from_cpu.h
 *  gpu_hierleap
 *
 *  Created by David Orendorff on 2/2/11.
 *  Copyright 2011 UC Irvine. All rights reserved.
 *
 */

#include <OpenCL/OpenCL.h>

#include "common.h"

#ifndef GPU_RANDGEN_FROM_CPU_H_
#define GPU_RANDGEN_FROM_CPU_H_

class GpuRandGenFromCpu : public GpuRandgen {
public:
	GpuRandGenFromCpu(int seed, cl_command_queue cmd_queue) : _cmd_queue(cmd_queue) {
		init_random(seed);
	}
	
	// Starts (and finishes) GPU command to generate RNs.
	// saves numToGen generated RNs to randomNums.
	// Assumes randomNums has been initialized to the appropriate size.
	// Note: this will be most efficient if: numToGen % (CA_WIDTH/4)==0
	virtual void generate_rands_in_gpu(cl_mem randomNumsStorage, int numToGen){
		
		unsigned int *nums = (unsigned int*)malloc(numToGen * sizeof(unsigned int));
		get_many_randints(nums, numToGen);
		int err = clEnqueueWriteBuffer(_cmd_queue, randomNumsStorage, CL_TRUE, 0, sizeof(unsigned int) * numToGen,
									   (void*)nums, 0, NULL, NULL);
		assert(err == CL_SUCCESS);
		clFinish(_cmd_queue);
		free(nums);
	}
	
	// Used for testing, this is not required if we do not need to transport rands from GPU
	virtual void get_many_randints(unsigned int *dst, int nRandsToGen) {
		for (int i = 0; i < nRandsToGen; i++) {
			dst[i] = urand_int(0, UINT_MAX);
		}
	}
	
private:
	cl_command_queue _cmd_queue;
};

#endif