
#include "sbml_interface.h"

#include <stdio.h>

#include <vector>

using namespace std;


#include <sbml/SBMLTypes.h> 

//#include "sbml_datastructs.h"

#include "../reactions/reaction.h"
#include "../state/species.h"

// Returns the index of this species in the model
int Model_species_idx(Model_t *m, const char *id){
	for (unsigned int i=0; i < Model_getNumSpecies(m); i++){
		Species_t *s = Model_getSpecies(m, i);
		if (!strcmp(Species_getId(s), id))
			return i;
	}
	return -1;
}

// Returns a vector of initial x-values for the species
// somewhat based on example code found in sbml library
vector<int> SBMLInterface::sbml_interface(char *filename, vector<hierleap::Reaction*> & reactions, vector<hierleap::Species*> & species){
	SBMLDocument_t *d;
	Model_t        *m;

	unsigned int level, version;

	d = readSBML(filename);

	SBMLDocument_printErrors(d, stdout);

	m = SBMLDocument_getModel(d);

	level   = SBMLDocument_getLevel  (d);
	version = SBMLDocument_getVersion(d);

	printf("\n");
	printf("File: %s (Level %u, version %u)\n", filename, level, version);

	if (m == NULL)
	{
	printf("No model present.");
	exit(1);
	}

	
	// My added code
	int numSpecies = Model_getNumSpecies(m);
	int numReactions = Model_getNumReactions(m);
	vector<int> initialX; initialX.resize(numSpecies);
	species.resize(numSpecies); reactions.resize(numReactions);

	// setup species
	for (int i=0; i < numSpecies; i++){
		Species_t *s = Model_getSpecies(m, i);
		int x0 = int(Species_getInitialAmount(s));
		const char *id = Species_getId(s);
		initialX[i] = x0;
		
		hierleap::Species *sp = new hierleap::Species((char*)id);
		sp->setTypeId(i);

		species[i] = sp;
	}
	
	// now set up reactions
	for (int r=0; r < numReactions; r++){
	
		Reaction_t *reaction = Model_getReaction(m, r);
		assert(reaction);
		
		// TODO: this is a hack to get what parameter is used by each reaction
		KineticLaw_t *kl = Reaction_getKineticLaw(reaction);
		const char* formula = KineticLaw_getFormula(kl);
		
		Parameter_t *p = NULL;
		double rate = 0;
		for (unsigned int k=0; k < Model_getNumParameters(m); k++){
			p = Model_getParameter(m,k);
			// this parameter is containd in formula
			if (strstr(formula, Parameter_getId(p))) {
				rate = Parameter_getValue(p);
			}
		}
		assert(rate != 0);
		
		hierleap::Reaction *rNew = new hierleap::Reaction(NULL, rate);
		
		// Now load in reactants and products
			
			for (unsigned int k=0; k < Reaction_getNumReactants(reaction); k++){
				SpeciesReference_t *sr = Reaction_getReactant(reaction, k);
				int s = Model_species_idx(m, SpeciesReference_getSpecies(sr));
				int stoic = int(SpeciesReference_getStoichiometry(sr));
				
				rNew->addReactant(species[s], abs(stoic));
			}
			
			for (unsigned int k=0; k < Reaction_getNumProducts(reaction); k++){
				SpeciesReference_t *sr = Reaction_getProduct(reaction, k);
				int s = Model_species_idx(m, SpeciesReference_getSpecies(sr));
				int stoic = int(SpeciesReference_getStoichiometry(sr));
				
				rNew->addProduct(species[s], abs(stoic));
			}
			reactions[r] = rNew;
		}

	  SBMLDocument_free(d);
	  return initialX;
}
